import os

import lbx

import star
import planet
import colony
import player
import starship

def compose_int(b0, b1, b2 = 0, b3 = 0):
    return b0 + (b1 << 8) + (b2 << 16) + (b3 << 24)
# end func get_int

class Moo2Savegame():
    """
    
    """

    def __init__(self, filename):
        """
        Loads the original MOO2 savegame
        """
        filesize = os.path.getsize(filename)
        savefile = open(filename, 'rb')
        self.__data = savefile.read(filesize)
        savefile.close()

    def parse_game(self):
        return {
            'name':				lbx.read_string(self.__data, 0x04, 23).lstrip(chr(0) + chr(1) + chr(2) + chr(3)),
            'end_of_turn_summary':		lbx.read_byte(self.__data, 0x2E),
            'end_of_turn_wait':			lbx.read_byte(self.__data, 0x2F),
            'random_events':			lbx.read_byte(self.__data, 0x30),
            'enemy_moves':			lbx.read_byte(self.__data, 0x31),
            'expanding_help':			lbx.read_byte(self.__data, 0x32),
            'auto_select_ships':		lbx.read_byte(self.__data, 0x33),
            'animations':			lbx.read_byte(self.__data, 0x34),
            'auto_select_colony':		lbx.read_byte(self.__data, 0x35),
            'show_relocation_lines':		lbx.read_byte(self.__data, 0x36),
            'show_gnn_report':			lbx.read_byte(self.__data, 0x37),
            'auto_delete_trade_good_housing':	lbx.read_byte(self.__data, 0x38),
            'auto_save_game':			lbx.read_byte(self.__data, 0x39),
            'show_only_serious_turn_summary':	lbx.read_byte(self.__data, 0x3A),
            'ship_initiative':			lbx.read_byte(self.__data, 0x3B),
        }
    # end func read_game

    def parse_galaxy(self):
        return {
            'size_factor':	lbx.read_byte(self.__data, 0x31be4),
            'width':            compose_int(ord(self.__data[0x31be9]), ord(self.__data[0x31bea])),
            'height':           compose_int(ord(self.__data[0x31beb]), ord(self.__data[0x31bec])),
            'stardate':		lbx.read_short_int(self.__data, 0x29),
        }
    # end func parse_galaxy

    def parse_hero_common_skills(self, type, skills_flags):
#        print "Moo2Savegame::parse_hero_common_skills ... type = %i, skills_flags = %i" % (type, skills_flags)
        print " * common_skills = %i" % skills_flags
        if type == 1:
            # Colony Leader
            if skills_flags & 1:
                print ""
            if skills_flags & 2:
                print ""
            if skills_flags & 4:
                print ""
            if skills_flags & 8:
                print ""
            if skills_flags & 16:
                print ""
            if skills_flags & 32:
                print ""
            if skills_flags & 64:
                print ""
            if skills_flags & 128:
                print ""
            if skills_flags & 256:
                print ""
            if skills_flags & 512:
                print ""
    # end func parse_hero_common_skills

    def parse_hero_skills(self, type, common_skills, special_skills):
#        print "Moo2Savegame::parse_hero_common_skills ... type = %i, skills_flags = %i" % (type, skills_flags)
#        print " * special_skills = %i" % special_skills
        skills = {}
        
        if common_skills & 1:
            skills['assasin']       = 2             # bit  0 = Assasin
        if common_skills & 2:
            skills['assasin']       = 3             # bit  1 = Assasin*

        if common_skills & 4:
            skills['commando']      = 2             # bit  2 = Commando
        if common_skills & 8:
            skills['commando']      = 3             # bit  3 = Commando*

        if common_skills & 16:
            skills['diplomat']      = 10            # bit  4 = Diplomat
        if common_skills & 32:
            skills['diplomat']      = 15            # bit  5 = Diplomat*

        if common_skills & 64:
            skills['famous']        = 60            # bit  6 = Famous
        if common_skills & 128:
            skills['famous']        = 90            # bit  7 = Famous*

        if common_skills & 256:
            skills['megawealth']    = 10            # bit  8 = Megawealth
        if common_skills & 512:
            skills['megawealth']    = 15            # bit  9 = Megawealth*

        if common_skills & 1024:
            skills['operations']    = 2             # bit 10 = Operations
        if common_skills & 2048:
            skills['operations']    = 3             # bit 11 = Operations*

        if common_skills & 4096:
            skills['researcher']    = 5             # bit 12 = Researcher
        if common_skills & 8192:
            skills['researcher']    = 7.5           # bit 13 = Researcher*

        if common_skills & 16384:
            skills['spy_master']    = 2             # bit 14 = Spy Master
        if common_skills & 32768:
            skills['spy_master']    = 3             # bit 15 = Spy Master*

        if common_skills & 65536:
            skills['telepath']      = 2             # bit 16 = Telepath
        if common_skills & 131072:
            skills['telepath']      = 3             # bit 17 = Telepath*

        if common_skills & 262144:
            skills['trader']        = 10            # bit 18 = Trader
        if common_skills & 524288:
            skills['trader']        = 15            # bit 19 = Trader*


        if type == 0:
            pass
        elif type == 1:
            # Colony Leader
            if special_skills & 1:
                skills['pollution_bonus'] = 10      # bit  0 = Environmentalist
            if special_skills & 2:
                skills['pollution_bonus'] = 15      # bit  1 = Environmentalist*

            if special_skills & 4:
                skills['farming_bonus'] = 10        # bit  2 = Farming Leader
            if special_skills & 8:
                skills['farming_bonus'] = 15        # bit  3 = Farming Leader*

            if special_skills & 16:
                skills['income_bonus'] = 10         # bit  4 = Financial Leader
            if special_skills & 32:
                skills['income_bonus'] = 15         # bit  5 = Financial Leader*

            if special_skills & 64:
                skills['instructor'] = 0.7          # bit  6 = Instructor
            if special_skills & 128:
                skills['instructor'] = 1.1          # bit  7 = Instructor*

            if special_skills & 256:
                skills['industry_bonus'] = 10       # bit  8 = Labor Leader
            if special_skills & 512:
                skills['industry_bonus'] = 15       # bit  9 = Labor Leader*

            if special_skills & 1024:
                skills['medicine'] = 10             # bit 10 = Medicine
            if special_skills & 2048:
                skills['medicine'] = 15             # bit 11 = Medicine*

            if special_skills & 4096:
                skills['research_bonus'] = 10       # bit 12 = Science Leader
            if special_skills & 8192:
                skills['research_bonus'] = 15       # bit 13 = Science Leader*

            if special_skills & 16384:
                skills['morale_bonus'] =  0         # bit 14 = Spiritual Leader
            if special_skills & 32768:
                skills['morale_bonus'] = 7.5        # bit 15 = Spiritual Leader*

            if special_skills & 65536:
                skills['tactics'] = 6               # bit 16 = Tactics
            if special_skills & 131072:
                skills['tactics'] = 9               # bit 17 = Tactics*

            return skills
    # end func parse_hero_special_skills

    def parse_heroes(self):
        HEROES_DATA_OFFSET	= 0x19a9b
        HERO_RECORD_SIZE	= 0x3b 		# = 59
        heroes = {}
        for i in range(67):
            hero_id = i
            offset = HEROES_DATA_OFFSET + (HERO_RECORD_SIZE * i)

#            print "hero @ %x" % offset

            type            = lbx.read_byte(self.__data, offset + 0x23)
            common_skills   = lbx.read_long_int(self.__data, offset + 0x26)
            special_skills  = lbx.read_long_int(self.__data, offset + 0x2a)

            heroes[hero_id] = {
                'id':               i,
#                'name':             data[offset:offset + 0x0f].rstrip(chr(0)).split(chr(0))[0],
                'name':             lbx.read_string(self.__data, offset, 15),
#                'title':            data[offset + 0x0f:offset + 0x23].rstrip(chr(0)).split(chr(0))[0],
                'title':            lbx.read_string(self.__data, offset, 35),
                'type':             type,
                'experience':       lbx.read_short_int(self.__data, offset + 0x24),
                'tech1':            lbx.read_byte(self.__data, offset + 0x2e),
                'tech2':            lbx.read_byte(self.__data, offset + 0x2f),
                'tech3':            lbx.read_byte(self.__data, offset + 0x30),
                'picture':          lbx.read_byte(self.__data, offset + 0x31),
                'skill_value':      lbx.read_short_int(self.__data, offset + 0x32),
                'level':            lbx.read_byte(self.__data, offset + 0x34),
                'location':         lbx.read_short_int(self.__data, offset + 0x35),
                'eta':              lbx.read_byte(self.__data, offset + 0x37),
                'level_up':         lbx.read_byte(self.__data, offset + 0x38),
                'status':           lbx.read_byte(self.__data, offset + 0x39),
                'player':           lbx.read_byte(self.__data, offset + 0x3a),
                'skills':           self.parse_hero_skills(type, common_skills, special_skills),
		'common_skills':    common_skills,
		'special_skills':   special_skills,
            }
#            if heroes[hero_id]['player'] == 0:
#                print "Moo2Savegame::parse_heroes ... hero_id = %i, name = %s" % (hero_id, heroes[hero_id]['name'])
#                print " * level = %i" % heroes[hero_id]['level']
#                print " * skill_value = %i" % heroes[hero_id]['skill_value']
#                print " * experience = %i" % heroes[hero_id]['experience']
#                print " * status = %i" % heroes[hero_id]['status']
            
#        i += 1
#        offset = 0x19a9b + (59 * i)
#        print "STOP hero @ %x" % offset
        return heroes
    # end func parse_heroes

    def parse_known_techs(self, offset):
        known_techs = []
        for i in range(203):               # original moo2 mas 203 usable technologies
#            offset2 = offset + 0x117 + i
            tech_status = ord(self.__data[offset + i])
 #           if tech_status and tech_status != 1:
#               print "tech_status = %i" % tech_status
            if tech_status == 3:
                known_techs.append(i + 1)
        return known_techs
    # end func parse_known_techs

    def parse_players(self):
        #
        # Players
        #
        PLAYERS_DATA_OFFSET	= 0x01aa0f
        PLAYER_RECORD_SIZE	= 3753

        pl = {}
        players = {}

#        print
#        print "=== Players ==="

        for i in range(8):

            player_id = i
            offset		= PLAYERS_DATA_OFFSET + (PLAYER_RECORD_SIZE * i)
#            print "player @ %x" % offset

#            players.append({
            players[player_id] = player.Player(player_id)
#            pl[player_id].import_from_moo2(self.__data[offset:offset + PLAYER_RECORD_SIZE])
            players[player_id].set_emperor_name(lbx.read_string(self.__data, offset, 15))
            players[player_id].set_race_name(lbx.read_string(self.__data, offset + 0x14, 36))
            players[player_id].set_picture(lbx.read_byte(self.__data, offset + 0x24))
            players[player_id].set_color(lbx.read_byte(self.__data, offset + 0x25))
            players[player_id].set_personality(ord(self.__data[offset + 0x26]))
            players[player_id].set_objective(ord(self.__data[offset + 0x27]))
            players[player_id].set_tax_rate(ord(self.__data[offset + 0x30]))
            players[player_id].set_bc(compose_int(ord(self.__data[offset + 0x31]), ord(self.__data[offset + 0x32]), ord(self.__data[offset + 0x33]), ord(self.__data[offset + 0x34])))
            players[player_id].set_total_frighters(compose_int(ord(self.__data[offset + 0x35]), ord(self.__data[offset + 0x36])))
            players[player_id].set_used_frighters(compose_int(ord(self.__data[offset + 0x37]), ord(self.__data[offset + 0x38])))
            players[player_id].set_command_points(compose_int(ord(self.__data[offset + 0x39]), ord(self.__data[offset + 0x3A])))
            players[player_id].set_industry(compose_int(ord(self.__data[offset + 0xA9]), ord(self.__data[offset + 0xAA])))
            players[player_id].set_research(compose_int(ord(self.__data[offset + 0xAB]), ord(self.__data[offset + 0xAC])))
            players[player_id].set_food(compose_int(ord(self.__data[offset + 0x0AF]), ord(self.__data[offset + 0x0B0])))
#            players[player_id].set_bc_income(compose_int(ord(self.__data[offset + 0x0B1]), ord(self.__data[offset + 0x0B2])))
            players[player_id].set_bc_income(lbx.read_signed_short_int(self.__data, offset + 0xB1))
            players[player_id].set_research_progress(compose_int(ord(self.__data[offset + 0x1EA]), ord(self.__data[offset + 0x1EB])))
            players[player_id].set_research_area(ord(self.__data[offset + 0x320]))
            players[player_id].set_research_item(ord(self.__data[offset + 0x321]))
            players[player_id].set_research_costs(0)
            players[player_id].set_racepicks({
                                        'goverment':		ord(self.__data[offset + 0x89E]),
                                        'population':		lbx.read_signed_byte(self.__data, offset + 0x89F),
                                        'farming':		lbx.read_signed_byte(self.__data, offset + 0x8A0),
                                        'industry':		lbx.read_signed_byte(self.__data, offset + 0x8A1),
                                        'science':		lbx.read_signed_byte(self.__data, offset + 0x8A2),
                                        'money':		lbx.read_signed_byte(self.__data, offset + 0x8A3),
                                        'ship_defense':		lbx.read_signed_byte(self.__data, offset + 0x8A4),
                                        'ship_attack':		lbx.read_signed_byte(self.__data, offset + 0x8A5),
                                        'ground_combat':	lbx.read_signed_byte(self.__data, offset + 0x8A6),
                                        'spying':		lbx.read_signed_byte(self.__data, offset + 0x8A7),
                                        'low_g':		ord(self.__data[offset + 0x8A8]),
                                        'high_g':		ord(self.__data[offset + 0x8A9]),
                                        'aquatic':		ord(self.__data[offset + 0x8AA]),
                                        'subterranean':		ord(self.__data[offset + 0x8AB]),
                                        'large_home_world':	ord(self.__data[offset + 0x8AC]),
                                        'rich_home_world':	ord(self.__data[offset + 0x8AD]),
                                        # where is poor_home_worlds?
                                        'artifacts_home_world':	ord(self.__data[offset + 0x8AE]),
                                        'cybernetic':		ord(self.__data[offset + 0x8AF]),
                                        'lithovore':		ord(self.__data[offset + 0x8B0]),
                                        'repulsive':		ord(self.__data[offset + 0x8B1]),
                                        'charismatic':		ord(self.__data[offset + 0x8B2]),
                                        'uncreative':		ord(self.__data[offset + 0x8B3]),
                                        'creative':		ord(self.__data[offset + 0x8B4]),
                                        'tolerant':		ord(self.__data[offset + 0x8B5]),
                                        'fantastic_traders':	ord(self.__data[offset + 0x8B6]),
                                        'telepathic':		ord(self.__data[offset + 0x8B7]),
                                        'lucky':		ord(self.__data[offset + 0x8B8]),
                                        'omniscience':		ord(self.__data[offset + 0x8B9]),
                                        'stealthy_ships':	ord(self.__data[offset + 0x8BA]),
                                        'trans_dimensional':	ord(self.__data[offset + 0x8BB]),
                                        'warlord':		ord(self.__data[offset + 0x8BC]),
                                    })
            players[player_id].set_known_techs(self.parse_known_techs(offset + 0x117))
            players[player_id].set_prototypes([])
            players[player_id].set_tributes([])
            for ii in range(6):
                offset2 = offset + 0x325 + (ii * 0x63)
                players[player_id].add_prototype(self.parse_ship_design(offset2))

            for ii in range(7):
                offset2 = offset + 0x649 + (ii)
                players[player_id].add_tribute(ord(self.__data[offset2]))
#            players[player_id].set_()
	    players[8] = player.Player(8)
	    players[8].set_race_name("Antareans")
	    players[8].set_emperor_name("Antarean")

	    players[9] = player.Player(8)
	    players[9].set_race_name("Orion")
	    players[9].set_emperor_name("Loknar")

	    players[10] = player.Player(10)
	    players[10].set_race_name("Space Amoeba")
	    players[10].set_emperor_name("Amoeba")

	    players[11] = player.Player(11)
	    players[11].set_race_name("Space Crystal")
	    players[11].set_emperor_name("Crystal")

	    players[12] = player.Player(12)
	    players[12].set_race_name("Space Dragon")
	    players[12].set_emperor_name("Dragon")

	    players[13] = player.Player(13)
	    players[13].set_race_name("Space Eeel")
	    players[13].set_emperor_name("Eel")

	    players[14] = player.Player(14)
	    players[14].set_race_name("Space Hydra")
	    players[14].set_emperor_name("Hydra")

	    for player_id in range(8, 15):
		players[player_id].set_bc(0)
		players[player_id].set_bc_income(0)
		players[player_id].set_research_area(0)
		players[player_id].set_research_item(0)
		players[player_id].set_food(0)
		players[player_id].set_research_costs(0)
		players[player_id].set_research_progress(0)
        return players
        """
        players.append({
            'race':		"Antareans",
            'emperor':	"?"
        })
        players.append({
            'race':		"Guardian",
            'emperor':	"?"
        })
        players.append({
            'race':		"Amoeba",
            'emperor':	"?"
        })
        players.append({
            'race':		"Crystal",
            'emperor':	"?"
        })
        players.append({
            'race':		"Dragon",
            'emperor':	"?"
        })
        players.append({
            'race':		"Eel",
            'emperor':	"?"
        })
        players.append({
            'race':		"Hydra",
            'emperor':	"?"
        })
        """
    #end func parse_players

    def parse_stars(self):
        #       http://www.spheriumnorth.com/orion-forum/nfphpbb/viewtopic.php?p=149
        SOLAR_SYSTEMS_COUNT_OFFSET	= 0x17ad1
        SOLAR_SYSTEMS_DATA_OFFSET	= 0x17ad3
        SOLAR_SYSTEM_RECORD_SIZE	= 0x71

        stars = {}
        c = ord(self.__data[SOLAR_SYSTEMS_COUNT_OFFSET])
        for star_id in range(c):
            offset = SOLAR_SYSTEMS_DATA_OFFSET + (SOLAR_SYSTEM_RECORD_SIZE * star_id)

            stars[star_id] = star.Star(star_id)
            stars[star_id].import_from_moo2(self.__data[offset:offset + SOLAR_SYSTEM_RECORD_SIZE])
        return stars

    def __old__parse_stars(self):

        for i in range(c):
            offset = SOLAR_SYSTEMS_DATA_OFFSET + (SOLAR_SYSTEM_RECORD_SIZE * i)
            systems = {}
#            print "system @ %x" % offset
            star_id = i
#            systems.append({
            systems[star_id] = {
                'id':			i,
                'name':     		lbx.read_string(self.__data, offset, 15),
                'x':        		lbx.read_short_int(self.__data, offset + 0x0f),
                'y':        		lbx.read_short_int(self.__data, offset + 0x11),
                'size':     		lbx.read_byte(self.__data, offset + 0x13),
                'owner':			lbx.read_byte(self.__data, offset + 0x14),		# primary owner
                'pict_type':		lbx.read_byte(self.__data, offset + 0x15),
                'class':    		lbx.read_byte(self.__data, offset + 0x16),
                'last_planet_selected':	[
                                                lbx.read_byte(self.__data, offset + 0x17),
                                                lbx.read_byte(self.__data, offset + 0x18),
                                                lbx.read_byte(self.__data, offset + 0x19),
                                                lbx.read_byte(self.__data, offset + 0x1A),
                                                lbx.read_byte(self.__data, offset + 0x1B),
                                                lbx.read_byte(self.__data, offset + 0x1C),
                                                lbx.read_byte(self.__data, offset + 0x1D),
                                                lbx.read_byte(self.__data, offset + 0x1E)
                                            ],
                'black_hole_blocks':	[
                                            ],
                '0x1f':		ord(self.__data[offset + 0x1f]),
                '0x20':		ord(self.__data[offset + 0x20]),
                '0x21':		ord(self.__data[offset + 0x21]),
                '0x22':		ord(self.__data[offset + 0x22]),
                '0x23':		ord(self.__data[offset + 0x23]),
                '0x24':		ord(self.__data[offset + 0x24]),
                '0x25':		ord(self.__data[offset + 0x25]),
                '0x26':		ord(self.__data[offset + 0x26]),
                '0x27':		ord(self.__data[offset + 0x27]),
                'special':  	ord(self.__data[offset + 0x28]),
                'wormhole': 	ord(self.__data[offset + 0x29]),
                '0x2a':		ord(self.__data[offset + 0x2a]),	# blockaded? ( 0 | 1 )
                '0x2b':		ord(self.__data[offset + 0x2b]),
                '0x2c':		ord(self.__data[offset + 0x2c]),
                '0x2d':		ord(self.__data[offset + 0x2d]),
                '0x2e':		ord(self.__data[offset + 0x2e]),
                '0x2f':		ord(self.__data[offset + 0x2f]),
                '0x30':		ord(self.__data[offset + 0x30]),
                '0x31':		ord(self.__data[offset + 0x31]),
                '0x32':		ord(self.__data[offset + 0x32]),
                'visited':		ord(self.__data[offset + 0x33]),	# bitmask as boleans for each player
                '0x34':		ord(self.__data[offset + 0x34]),
                '0x35':		ord(self.__data[offset + 0x35]),
                '0x36':		ord(self.__data[offset + 0x36]),
                '0x37':		ord(self.__data[offset + 0x37]),
                '0x38':		ord(self.__data[offset + 0x38]),
                'indictor': 	ord(self.__data[offset + 0x39]),	# 0 = none, 1-8 = owner player 0-7  ( http://code.google.com/p/moo2x/wiki/star )
                '0x3a':		ord(self.__data[offset + 0x3a]),
                '0x3b':		ord(self.__data[offset + 0x3b]),
                '0x3c':		ord(self.__data[offset + 0x3c]),
                '0x3d':		ord(self.__data[offset + 0x3d]),
                '0x3e':		ord(self.__data[offset + 0x3e]),
                'artemis':  	ord(self.__data[offset + 0x3f]),	# 0 = none, 1-8=owner player 0-7 ( http://code.google.com/p/moo2x/wiki/star )
                '0x40':		ord(self.__data[offset + 0x40]),	# Star has dimensional portal ?
                '0x41':		ord(self.__data[offset + 0x41]),
                '0x42':		ord(self.__data[offset + 0x42]),
                '0x43':		ord(self.__data[offset + 0x43]),
                '0x44':		ord(self.__data[offset + 0x44]),
                '0x45':		ord(self.__data[offset + 0x45]),
                '0x46':		ord(self.__data[offset + 0x46]),
                '0x47':		ord(self.__data[offset + 0x47]),
                '0x48':		ord(self.__data[offset + 0x48]),
                '0x49':		ord(self.__data[offset + 0x49]),
                'objects':   	[
                                        compose_int(ord(self.__data[offset + 0x4a]), ord(self.__data[offset + 0x4b])),
                                        compose_int(ord(self.__data[offset + 0x4c]), ord(self.__data[offset + 0x4d])),
                                        compose_int(ord(self.__data[offset + 0x4e]), ord(self.__data[offset + 0x4f])),
                                        compose_int(ord(self.__data[offset + 0x50]), ord(self.__data[offset + 0x51])),
                                        compose_int(ord(self.__data[offset + 0x52]), ord(self.__data[offset + 0x53]))
                                    ],
    #		??? Relocation star id (0-7 player #, not sure about size of array. )
                '0x54':		ord(self.__data[offset + 0x54]),
                '0x55':		ord(self.__data[offset + 0x55]),
                '0x56':		ord(self.__data[offset + 0x56]),
                '0x57':		ord(self.__data[offset + 0x57]),
                '0x58':		ord(self.__data[offset + 0x58]),
                '0x59':		ord(self.__data[offset + 0x59]),
                '0x5a':		ord(self.__data[offset + 0x5a]),
                '0x5b':		ord(self.__data[offset + 0x5b]),
                '0x5c':		ord(self.__data[offset + 0x5c]),
                '0x5d':		ord(self.__data[offset + 0x5d]),
                '0x5e':		ord(self.__data[offset + 0x5e]),
                '0x5f':		ord(self.__data[offset + 0x5f]),
                '0x60':		ord(self.__data[offset + 0x60]),
                '0x61':		ord(self.__data[offset + 0x61]),
                '0x62':		ord(self.__data[offset + 0x62]),
                '0x63':		ord(self.__data[offset + 0x63]),
    #		unknown:
                '0x64':		ord(self.__data[offset + 0x64]),
                '0x65':		ord(self.__data[offset + 0x65]),
                '0x66':		ord(self.__data[offset + 0x66]),
                '0x67':		ord(self.__data[offset + 0x67]),
                '0x68':		ord(self.__data[offset + 0x68]),
                '0x69':		ord(self.__data[offset + 0x69]),
                '0x6a':		ord(self.__data[offset + 0x6a]),
                '0x6b':		ord(self.__data[offset + 0x6b]),
                '0x6c':		ord(self.__data[offset + 0x6c]),
                '0x6d':		ord(self.__data[offset + 0x6d]),
                '0x6e':		ord(self.__data[offset + 0x6e]),
                '0x6f':		ord(self.__data[offset + 0x6f]),
                '0x70':		ord(self.__data[offset + 0x70])
                }
#        i += 1
#        offset = 0x17ad3 + (SOLAR_SYSTEM_RECORD_SIZE * i)
#        print "STOP system @ %x" % offset
    #    blah
        return systems
    # end func parse_stars

    def parse_planets(self, stars):
        PLANETS_COUNT_OFFSET	= 0x162e7
        PLANETS_DATA_OFFSET		= 0x162e9
        PLANET_RECORD_SIZE		= 0x11
        planets = {}
#        pl = {}

        c = 0
        for star_id in stars:
            star = stars[star_id]
            for o in star.get_objects():
                if o != 0xffff:
                    c += 1

        c = compose_int(ord(self.__data[PLANETS_COUNT_OFFSET]), ord(self.__data[PLANETS_COUNT_OFFSET + 1]))
#        c = 250

#        print "Reading planetary objects ..."
#        print "    count: %i" % c

        for i in range(c):
            planet_id = i
            offset = PLANETS_DATA_OFFSET + (PLANET_RECORD_SIZE * i)
    #        system_number = ord(self.__data[offset + 0x02])
    #        position = ord(self.__data[offset + 0x03])
#            planets.append({
            planets[planet_id] = planet.Planet(planet_id)
            planets[planet_id].import_from_moo2(self.__data[offset:offset + PLANET_RECORD_SIZE])
#            pl[planet_id] = {
#                'planet_id':        planet_id,
#                'colony':           lbx.read_short_int(self.__data, offset),	# 0xffff = no colony here
#                'parent_star':      lbx.read_byte(self.__data, offset + 0x02),
#                'position':         lbx.read_byte(self.__data, offset + 0x03),
#                'type':             lbx.read_byte(self.__data, offset + 0x04),
#                'size':             lbx.read_byte(self.__data, offset + 0x05),
#                'gravity':          lbx.read_byte(self.__data, offset + 0x06),
#                'group':            lbx.read_byte(self.__data, offset + 0x07),   # not used
#                'terrain':          lbx.read_byte(self.__data, offset + 0x08),
#                'picture':          lbx.read_byte(self.__data, offset + 0x09),   # Background image on colony screen (0-5=image in planets.lbx)
#                'minerals':         lbx.read_byte(self.__data, offset + 0x0a),
#                'foodbase':         lbx.read_byte(self.__data, offset + 0x0b),
#                'terraformations':  lbx.read_byte(self.__data, offset + 0x0c),
#                'max_farms':        lbx.read_byte(self.__data, offset + 0x0d),   # unknown (Initial value is based on Planet Size but changes if colonized), 2=tiny, 4=small, 5=med, 7=large, A=huge
#                'max_population':   lbx.read_byte(self.__data, offset + 0x0e),
#                'special':          lbx.read_byte(self.__data, offset + 0x0f),
#                'flags':            lbx.read_byte(self.__data, offset + 0x10)    # (bit 2 = Soil Enrichment)
#            }
        return planets
    # end func parse_planets

    def parse_colonies(self):
        COLONIES_DATA_OFFSET	= 0x0025d
        COLONY_RECORD_SIZE		= 361
        c = compose_int(ord(self.__data[0x0000025b]), ord(self.__data[0x0000025c]))

#        print "Number of colonies to read: %i" % c

        colonies = {}

        for colony_id in range(c):
#            print("Moo2Savegame::parse_colonies() ... colony_id = %i" % colony_id)
            offset	= COLONIES_DATA_OFFSET + (COLONY_RECORD_SIZE * colony_id)
            colonies[colony_id] = colony.Colony(colony_id)
            colonies[colony_id].import_from_moo2(self.__data[offset:offset + COLONY_RECORD_SIZE])
#            print
        return colonies
    # end func parse_colonies

    def parse_ship_design(self, offset):
        MAX_WEAPONS = 8
        weapons = []
        for i in range(MAX_WEAPONS):
            offset2 = offset + 0x1C + (8 * i)
            weapon = lbx.read_byte(self.__data, offset2)
            if (weapon > 0):
                weapons.append({
                    'weapon':		weapon,
                    'count':		lbx.read_byte(self.__data, offset2 + 0x02),
                    'current_count':	lbx.read_byte(self.__data, offset2 + 0x03),
                    'arc':		lbx.read_byte(self.__data, offset2 + 0x04),
                    'beam_mods':	lbx.read_byte(self.__data, offset2 + 0x05),
                    'missile_mods':	lbx.read_byte(self.__data, offset2 + 0x06),
                    'ammo':		lbx.read_byte(self.__data, offset2 + 0x07)
                })
        return {
                'name':                 lbx.read_string(self.__data, offset, 16),
                'size':         	lbx.read_byte(self.__data, offset + 0x10),
                'type':                 lbx.read_byte(self.__data, offset + 0x11),
                'shield':		lbx.read_byte(self.__data, offset + 0x12),
                'drive':		lbx.read_byte(self.__data, offset + 0x13),
                'speed':		lbx.read_byte(self.__data, offset + 0x14),
                'computer':		lbx.read_byte(self.__data, offset + 0x15),
                'armor':		lbx.read_byte(self.__data, offset + 0x16),
                'special_devices':	self.__data[offset + 0x17:offset + 0x1C],	# char  special_device_flags[(MAX_SPECIALS+7)/8];
                'weapons':		weapons,
                'picture':		lbx.read_byte(self.__data, offset + 0x5C),
                'builder':		lbx.read_byte(self.__data, offset + 0x5D),		# or previous owner?
                'cost':                 lbx.read_short_int(self.__data, offset + 0x5E),
                'combat_speed':         lbx.read_byte(self.__data, offset + 0x60),
                'build_date':           lbx.read_short_int(self.__data, offset + 0x61)
        }
    # end func parse_ship_design

    def parse_one_ship(self, ship_id):
        SAVE_SHIPS_OFFSET         = 0x21f58
        SAVE_SHIP_RECORD_SIZE     = 0x81        # = 129
        offset          = SAVE_SHIPS_OFFSET + (SAVE_SHIP_RECORD_SIZE * ship_id)
        data = self.__data[offset:offset + SAVE_SHIP_RECORD_SIZE]

        ship = starship.Starship(ship_id)

        ship.set_design(self.parse_ship_design(offset))
        ship.set_owner(lbx.read_byte(data, 0x63))
        ship.set_status(lbx.read_byte(data, 0x64))                               # 0 = orbitting, 2 = traveling, 5 = destroyed?
        ship.set_destination(lbx.read_short_int(data, 0x65) % 500)                             # destination star
        ship.set_x(lbx.read_short_int(data, 0x67))
        ship.set_y(lbx.read_short_int(data, 0x69))
        ship.set_group_has_navigator(lbx.read_byte(data, 0x6B))
        ship.set_travelling_speed(lbx.read_byte(data, 0x6C))     # possibly less than ftl_type
        ship.set_turns_left(lbx.read_byte(data, 0x6D))           # until arrival
        ship.set_shield_damage_percent(lbx.read_byte(data, 0x6E))
        ship.set_drive_damage_percent(lbx.read_byte(data, 0x6F))
        ship.set_computer_damage(lbx.read_byte(data, 0x70))
        ship.set_crew_quality(lbx.read_byte(data, 0x71))
        ship.set_crew_experience(lbx.read_short_int(data, 0x72))
        ship.set_officer_id(lbx.read_short_int(data, 0x74))
        ship.set_special_device_damage(data[0x76:0x7B]) # bit flag array
        ship.set_armor_damage(lbx.read_short_int(data, 0x7B))
        ship.set_structural_damage(lbx.read_short_int(data, 0x7D))
        ship.set_mission(lbx.read_byte(data, 0x7F))              # used for AI
        ship.set_just_built(lbx.read_byte(data, 0x80))

        return ship

    def parse_ships(self):
        SAVE_SHIP_COUNT_OFFSET    = 0x21f56
        SAVE_SHIPS_OFFSET         = 0x21f58
        SAVE_SHIP_RECORD_SIZE     = 0x81        # = 129
    #    SAVE_SHIP_WEAPONS_OFFSET  = 0x1c
    #    SAVE_SHIP_SPECIALS_OFFSET = 0x17

        c = compose_int(ord(self.__data[SAVE_SHIP_COUNT_OFFSET]), ord(self.__data[SAVE_SHIP_COUNT_OFFSET + 1]))
        ships = {}
        for ship_id in range(c):
#            print "ship @ %x" % offset
            ships[ship_id] = self.parse_one_ship(ship_id)
        return ships
    # end func parse_ships

    def set_rules(self, rules):
        self.__rules = rules
    # end func set_rules

    def init_stars(self):
        self.__stars_by_coords = {}

        for star_id in self.__stars:
            star = self.__stars[star_id]
            k = "%i:%i" % (star['x'], star['y'])
            self.__stars_by_coords[k] = star
            num = 0
            for object in star['objects']:
#               print "object: %i" % object
                if object != 0xffff:
                    num += 1
                    self.__planets[object]['num'] = num
    # end func init_stars

    def init_heroes(self):
        self.__players_heroes = {}
        for hero in self.__heroes:
            if hero['player'] != 0xFF:
                if not self.__players_heroes.has_key(hero['player']):
                    self.__players_heroes[hero['player']] = {}
                self.__players_heroes[hero['player']][hero['id']] = hero
#        print self.__players_heroes
    # end func init_heroes



TECH_NONE	= 0
TECH_UNKNOWN	= 1
TECH_KNOWN	= 3

TECH_TABLE = {
    0:  {
	'name':		"",
	'area':		0,
    },
    1:	{
	'name':		"Achilles Targeting Unit",
	'area':		49,
    },
    2:	{
	'name':		"Adamantium Armor",
	'area':		48,
    },
    3:	{
	'name':		"Advanced City Planning",
	'area':		42,
    },
    4:	{
	'name':		"Advanced Damage Control",
	'area':		63,
    },
    5:	{
	'name':		"Alien Management Center",
	'area':		73,
    },
    6:	{
	'name':		"Android Farmers",
	'area':		24,
    },
    7:	{
	'name':		"Android Scientists",
	'area':		24,
    },
    8:	{
	'name':		"Android Workers",
	'area':		24,
    },
    9:	{
	'name':		"Anti-Gravity Harness",
	'area':		36,
    },
    10:	{
	'name':		"Anti-Matter Bomb",
	'area':		13,
    },
    11:	{
	'name':		"Anti-Matter Drive",
	'area':		13,
    },
    12:	{
	'name':		"Anti-Matter Torpedoes",
	'area':		13,
    },
    13:	{
	'name':		"Anti-Missile Rockets",
	'area':		4,
    },
    14:	{
	'name':		"Armor Barracks",
	'area':		20,
    },
    15:	{
	'name':		"Artemis System Net",
	'area':		58,
    },
    16:	{
	'name':		"Planet Construction",
	'area':		8,
    },
    17:	{
	'name':		"Assault Shuttles",
	'area':		63,
    },
    18:	{
	'name':		"Astro University",
	'area':		12,
    },
    19:	{
	'name':		"Atmospheric Renewer",
	'area':		47,
    },
    20:	{
	'name':		"Augmented Engines",
	'area':		5,
    },
    21:	{
	'name':		"Autolab",
	'area':		25,
    },
    22:	{
	'name':		"Automated Factories",
	'area':		3,
    },
    23:	{
	'name':		"Automated Repair Unit",
	'area':		8,
    },
    24:	{
	'name':		"Battleoids",
	'area':		19,
    },
    25:	{
	'name':		"Battle Pods",
	'area':		21,
    },
    26:	{
	'name':		"Battle Scanner",
	'area':		66,
    },
    27:	{
	'name':		"Battlestation",
	'area':		62,
    },
    28:	{
	'name':		"Bio-Terminator",
	'area':		17,
    },
    29:	{
	'name':		"Biomorphic Fungi",
	'area':		70,
    },
    30:	{
	'name':		"Black Hole Generator",
	'area':		0,
    },
    31:	{
	'name':		"Bomber Bays",
	'area':		11,
    },
    32:	{
	'name':		"Capitol",
	'area':		0,
    },
    33:	{
	'name':		"Class I Shield",
	'area':		7,
    },
    34:	{
	'name':		"Class III Shield",
	'area':		45,
    },
    35:	{
	'name':		"Class V Shield",
	'area':		64,
    },
    36:	{
	'name':		"Class VII Shield",
	'area':		61,
    },
    37:	{
	'name':		"Class X Shield",
	'area':		68,
    },
    38:	{
	'name':		"Cloaking Device",
	'area':		26,
    },
    39:	{
	'name':		"Cloning Center",
	'area':		1,
    },
    40:	{
	'name':		"Colony Base",
	'area':		0,
    },
    41:	{
	'name':		"Colony Ship",
	'area':		23,
    },
    42:	{
	'name':		"Confederation",
	'area':		0,
    },
    43:	{
	'name':		"Cyber-Security Link",
	'area':		14,
    },
    44:	{
	'name':		"Cybertronic Computer",
	'area':		25,
    },
    45:	{
	'name':		"Damper Field",
	'area':		0,
    },
    46:	{
	'name':		"Dauntless Guidance System",
	'area':		56,
    },
    47:	{
	'name':		"Death Ray",
	'area':		0,
    },
    48:	{
	'name':		"Death Spores",
	'area':		1,
    },
    49:	{
	'name':		"Deep Core Mining",
	'area':		67,
    },
    50:	{
	'name':		"Core Waste Dumps",
	'area':		67,
    },
    51:	{
	'name':		"Deuterium Fuel Cells",
	'area':		9,
    },
    52:	{
	'name':		"Dimensional Portal",
	'area':		51,
    },
    53:	{
	'name':		"Displacement Device",
	'area':		71,
    },
    54:	{
	'name':		"Disrupter Cannon",
	'area':		51,
    },
    55:	{
	'name':		"Doom Star Construction",
	'area':		58,
    },
    56:	{
	'name':		"Reinforced Hull",
	'area':		4,
    },
    57:	{
	'name':		"ECM Jammer",
	'area':		7,
    },
    58:	{
	'name':		"Electronic Computer",
	'area':		28,
    },
    59:	{
	'name':		"Emissions Guidance System",
	'area':		14,
    },
    60:	{
	'name':		"Energy Absorber",
	'area':		37,
    },
    61:	{
	'name':		"Biospheres",
	'area':		18,
    },
    62:	{
	'name':		"Evolutionary Mutation",
	'area':		70,
    },
    63:	{
	'name':		"Extended Fuel Tanks",
	'area':		22,
    },
    64:	{
	'name':		"Fast Missile Racks",
	'area':		63,
    },
    65:	{
	'name':		"Federation",
	'area':		0,
    },
    66:	{
	'name':		"Fighter Bays",
	'area':		4,
    },
    67:	{
	'name':		"Fighter Garrison",
	'area':		20,
    },
    68:	{
	'name':		"Food Replicators",
	'area':		46,
    },
    69:	{
	'name':		"Freighters",
	'area':		55,
    },
    70:	{
	'name':		"Fusion Beam",
	'area':		31,
    },
    71:	{
	'name':		"Fusion Bomb",
	'area':		5,
    },
    72:	{
	'name':		"Fusion Drive",
	'area':		5,
    },
    73:	{
	'name':		"Fusion Rifle",
	'area':		31,
    },
    74:	{
	'name':		"Gaia Transformation",
	'area':		70,
    },
    75:	{
	'name':		"Galactic Currency Exchange",
	'area':		32,
    },
    76:	{
	'name':		"Galactic Cybernet",
	'area':		33,
    },
    77:	{
	'name':		"Galactic Unification",
	'area':		0,
    },
    78:	{
	'name':		"Gauss Cannon",
	'area':		64,
    },
    79:	{
	'name':		"Graviton Beam",
	'area':		16,
    },
    80:	{
	'name':		"Gyro Destabilizer",
	'area':		36,
    },
    81:	{
	'name':		"Hard Shields",
	'area':		26,
    },
    82:	{
	'name':		"Heavy Armor",
	'area':		3,
    },
    83:	{
	'name':		"Heavy Fighter Bays",
	'area':		42,
    },
    84:	{
	'name':		"Heightened Intelligence",
	'area':		30,
    },
    85:	{
	'name':		"High Energy Focus",
	'area':		37,
    },
    86:	{
	'name':		"Holo Simulator",
	'area':		60,
    },
    87:	{
	'name':		"Hydroponic Farms",
	'area':		18,
    },
    88:	{
	'name':		"Hyper Drive",
	'area':		38,
    },
    89:	{
	'name':		"MegaFluxers",
	'area':		37,
    },
    90:	{
	'name':		"Hyper-X Capacitors",
	'area':		38,
    },
    91:	{
	'name':		"Hyperspace Communications",
	'area':		39,
    },
    92:	{
	'name':		"Imperium",
	'area':		6,
    },
    93:	{
	'name':		"Inertial Nullifier",
	'area':		71,
    },
    94:	{
	'name':		"Inertial Stabilizer",
	'area':		36,
    },
    95:	{
	'name':		"Interphased Drive",
	'area':		40,
    },
    96:	{
	'name':		"Ion Drive",
	'area':		41,
    },
    97:	{
	'name':		"Ion Pulse Cannon",
	'area':		41,
    },
    98:	{
	'name':		"Iridium Fuel Cells",
	'area':		47,
    },
    99:	{
	'name':		"Jump Gate",
	'area':		65,
    },
    100:	{
	'name':		"Laser Cannon",
	'area':		57,
    },
    101:	{
	'name':		"Laser Rifle",
	'area':		57,
    },
    102:	{
	'name':		"Lightning Field",
	'area':		72,
    },
    103:	{
	'name':		"Marine Barracks",
	'area':		0,
    },
    104:	{
	'name':		"Mass Driver",
	'area':		7,
    },
    105:	{
	'name':		"Mauler Device",
	'area':		39,
    },
    106:	{
	'name':		"Merculite Missile",
	'area':		2,
    },
    107:	{
	'name':		"Microbiotics",
	'area':		34,
    },
    108:	{
	'name':		"Microlite Construction",
	'area':		53,
        'worker_bonus': +1,
    },
    109:	{
	'name':		"Outpost Ship",
	'area':		23,
    },
    110:	{
	'name':		"Moleculartronic Computer",
	'area':		49,
    },
    111:	{
	'name':		"Multi-Wave Ecm Jammer",
	'area':		64,
    },
    112:	{
	'name':		"Multi-Phased Shields",
	'area':		52,
    },
    113:	{
	'name':		"Nano Disassemblers",
	'area':		53,
    },
    114:	{
	'name':		"Neural Scanner",
	'area':		15,
    },
    115:	{
	'name':		"Neutron Blaster",
	'area':		54,
    },
    116:	{
	'name':		"Neutron Scanner",
	'area':		54,
    },
    117:	{
	'name':		"Neutronium Armor",
	'area':		50,
    },
    118:	{
	'name':		"Neutronium Bomb",
	'area':		40,
    },
    119:	{
	'name':		"Nuclear Bomb",
	'area':		55,
    },
    120:	{
	'name':		"Nuclear Drive",
	'area':		55,
    },
    121:	{
	'name':		"Nuclear Missile",
	'area':		22,
    },
    122:	{
	'name':		"Optronic Computer",
	'area':		56,
    },
    123:	{
	'name':		"Particle Beam",
	'area':		0,
    },
    124:	{
	'name':		"Personal Shield",
	'area':		27,
    },
    125:	{
	'name':		"Phase Shifter",
	'area':		0,
    },
    126:	{
	'name':		"Phasing Cloak",
	'area':		68,
    },
    127:	{
	'name':		"Phasor",
	'area':		52,
    },
    128:	{
	'name':		"Phasor Rifle",
	'area':		52,
    },
    129:	{
	'name':		"Planetary Barrier Shield",
	'area':		68,
    },
    130:	{
	'name':		"Planetary Flux Shield",
	'area':		61,
    },
    131:	{
	'name':		"Planetary Gravity Generator",
	'area':		16,
    },
    132:	{
	'name':		"Planetary Missile Base",
	'area':		3,
    },
    133:	{
	'name':		"Ground Batteries",
	'area':		19,
    },
    134:	{
	'name':		"Planetary Radiation Shield",
	'area':		45,
    },
    135:	{
	'name':		"Planetary Stock Exchange",
	'area':		43,
    },
    136:	{
	'name':		"Planetary Supercomputer",
	'area':		60,
    },
    137:	{
	'name':		"Plasma Cannon",
	'area':		59,
    },
    138:	{
	'name':		"Plasma Rifle",
	'area':		59,
    },
    139:	{
	'name':		"Plasma Torpedoes",
	'area':		40,
    },
    140:	{
	'name':		"Plasma Web",
	'area':		59,
    },
    141:	{
	'name':		"Pleasure Dome",
	'area':		49,
    },
    142:	{
	'name':		"Pollution Processor",
	'area':		2,
    },
    143:	{
	'name':		"Positronic Computer",
	'area':		60,
    },
    144:	{
	'name':		"Powered Armor",
	'area':		62,
    },
    145:	{
	'name':		"Pulse Rifle",
	'area':		0,
    },
    146:	{
	'name':		"Proton Torpedoes",
	'area':		38,
    },
    147:	{
	'name':                             "Psionics",
	'area':                             30,
        'government_morale_bonus':          [0, 0, 10, 10, 0, 0, 0], # percent
    },
    148:	{
	'name':		"Pulsar",
	'area':		72,
    },
    149:	{
	'name':		"Pulson Missile",
	'area':		47,
    },
    150:	{
	'name':		"Quantum Detonator",
	'area':		0,
    },
    151:	{
	'name':		"Rangemaster Unit",
	'area':		14,
    },
    152:	{
	'name':		"Recyclotron",
	'area':		8,
    },
    153:	{
	'name':		"Reflection Field",
	'area':		0,
    },
    154:	{
	'name':		"Robotic Factory",
	'area':		11,
    },
    155:	{
	'name':		"Research Laboratory",
	'area':		56,
    },
    156:	{
	'name':		"Robo-Miners",
	'area':		62,
    },
    157:	{
	'name':		"Space Scanner",
	'area':		57,
    },
    158:	{
	'name':		"Scout Lab",
	'area':		15,
    },
    159:	{
	'name':		"Security Stations",
	'area':		15,
    },
    160:	{
	'name':		"Sensors",
	'area':		39,
    },
    161:	{
	'name':		"Shield Capacitors",
	'area':		41,
    },
    162:	{
	'name':		"Soil Enrichment",
	'area':		1,
    },
    163:	{
	'name':		"Space Academy",
	'area':		10,
    },
    164:	{
	'name':		"Spaceport",
	'area':		20,
    },
    165:	{
	'name':		"Spatial Compressor",
	'area':		0,
    },
    166:	{
	'name':		"Spy Network",
	'area':		0,
    },
    167:	{
	'name':		"Standard Fuel Cells",
	'area':		22,
    },
    168:	{
	'name':		"Star Base",
	'area':		0,
    },
    169:	{
	'name':		"Star Fortress",
	'area':		42,
    },
    170:	{
	'name':		"Star Gate",
	'area':		69,
    },
    171:	{
	'name':		"Stasis Field",
	'area':		26,
    },
    172:	{
	'name':		"Stealth Field",
	'area':		27,
    },
    173:	{
	'name':		"Stealth Suit",
	'area':		27,
    },
    174:	{
	'name':		"Stellar Converter",
	'area':		69,
    },
    175:	{
	'name':		"Structural Analyzer",
	'area':		25,
    },
    176:	{
	'name':		"Sub-Space Communications",
	'area':		65,
    },
    177:	{
	'name':		"Sub-Space Teleporter",
	'area':		71,
    },
    178:	{
	'name':		"Subterranean Farms",
	'area':		44,
    },
    179:	{
	'name':		"Survival Pods",
	'area':		21,
    },
    180:	{
	'name':		"Tachyon Communications",
	'area':		66,
    },
    181:	{
	'name':		"Tachyon Scanner",
	'area':		66,
    },
    182:	{
	'name':		"Telepathic Training",
	'area':		34,
    },
    183:	{
	'name':		"Terraforming",
	'area':		35,
    },
    184:	{
	'name':		"Thorium Fuel Cells",
	'area':		48,
    },
    185:	{
	'name':		"Time Warp Facilitator",
	'area':		69,
    },
    186:	{
	'name':		"Titan Construction",
	'area':		19,
    },
    187:	{
	'name':		"Titanium Armor",
	'area':		22,
    },
    188:	{
	'name':		"Tractor Beam",
	'area':		16,
    },
    189:	{
	'name':		"Transport",
	'area':		23,
    },
    190:	{
	'name':		"Transporters",
	'area':		46,
    },
    191:	{
	'name':		"Tritanium Armor",
	'area':		9,
    },
    192:	{
	'name':		"Troop Pods",
	'area':		21,
    },
    193:	{
	'name':		"Universal Antidote",
	'area':		17,
    },
    194:	{
	'name':		"Urridium Fuel Cells",
	'area':		50,
    },
    195:	{
	'name':             "Virtual Reality Network",
	'area':             33,
        'morale_bonus':     20, # percent
    },
    196:	{
	'name':		"Warp Dissipater",
	'area':		45,
    },
    197:	{
	'name':		"Warp Interdictor",
	'area':		72,
    },
    198:	{
	'name':		"Weather Control System",
	'area':		44,
    },
    199:	{
	'name':		"Wide Area Jammer",
	'area':		61,
    },
    200:	{
	'name':		"Xeno Psychology",
	'area':		73,
    },
    201:	{
	'name':		"Xentronium Armor",
	'area':		0,
    },
    202:	{
	'name':		"Zeon Missile",
	'area':		50,
    },
    203:	{
	'name':		"Zortrium Armor",
	'area':		53,
    },
    204:	{
	'name':		"Biology I",
	'area':		75,
    },
    205:	{
	'name':		"Power I",
	'area':		76,
    },
    206:	{
	'name':		"Physics I",
	'area':		77,
    },
    207:	{
	'name':		"Construction I",
	'area':		78,
    },
    208:	{
	'name':		"Fields I",
	'area':		79,
    },
    209:	{
	'name':		"Chemistry I",
	'area':		80,
    },
    210:	{
	'name':		"Computers I",
	'area':		81,
    },
    211:	{
	'name':		"Sociology I",
	'area':		82,
    },
}



import pygame

import get_input

#import formulas

from _game_constants import *

import colony_screen

from _buildings import *
#from _technology import *

#
#       DRAW
#

def list_available_buildings(rules_buildings, colony_obj, known_techs):
    replaced_buildings = []
    for b_id in colony_obj.list_buildings():
        if rules_buildings[b_id].has_key("replaces"):
            print rules_buildings[b_id]['replaces']
            for rep_b_id in rules_buildings[b_id]['replaces']:
                replaced_buildings.append(rep_b_id)
        

    available_buildings = {}
    for b_id in rules_buildings:
        b_rec = rules_buildings[b_id]
        if b_rec['tech']:
            # knows required technology and not built
            if (b_rec['tech'] in known_techs) and (not colony_obj.has_building(b_id)) and (not b_id in replaced_buildings):
    		available_buildings[b_id] = b_rec
    return available_buildings
# end func list_available_buildings

def draw(GAME, star, planet, colony):
    GUI		= GAME['GUI']
    DISPLAY     = GAME['DISPLAY']
    IMAGES      = GAME['IMAGES']
    PALETTES	= GAME['PALETTES']
    FONTS       = GAME['FONTS']

    DATA        = GAME['DATA']
    RULES       = DATA['rules']
    PROTOTYPES  = DATA['prototypes']
    ME          = DATA['me']

    colony_screen.draw_planet_background(GUI, DISPLAY, IMAGES, PALETTES, planet.get_terrain(), planet.get_picture())

    shadow = pygame.Surface((640, 480))

#    shadow.fill((0, 0, 0))
#    shadow.fill((8, 8, 20))
    shadow.fill((28, 32, 44))
    shadow.set_alpha(180)
    DISPLAY.blit(shadow, (0, 0))

#    shadow.fill((255, 255, 255))
#    shadow.set_alpha(100)
#    DISPLAY.blit(shadow, (0, 0))


#    c1 = (0, 0, 0, 128)
#    c1 = (8, 8, 20, 128)
    c1 = (28, 32, 44, 128)

    for y in range(0, 480, 2):
        pygame.draw.line(DISPLAY, c1, (0, y), (639, y), 1)

#    for x in range(0, 640, 2):
#        pygame.draw.line(DISPLAY, c1, (x, 0), (x, 479), 1)

    DISPLAY.blit(IMAGES['COLONY_BUILD_SCREEN']['screen'], (0, 0))

#    buildings = colony.list_buildings()
    print("")
    print("=== Available Buildings: ===")
    available_buildings = list_available_buildings(RULES['buildings'], colony, ME.get_known_techs())
    for b_id in available_buildings:
	b_rec = available_buildings[b_id]
	print(b_id, b_rec)

    print("=== /Available Buildings ===")
    print("")

    yy = 0
    for b_id in available_buildings:
	label_surface = FONTS['font_12_bold'].render(available_buildings[b_id]['name'], 1, (0xE4, 0x88, 0x20))
	DISPLAY.blit(label_surface, (11, 20 + yy))
	yy += 14


    print("")
    print("=== Prototypes: ===")
    yy = 0
    for i in range(5):
        prototype = PROTOTYPES[i]
        print("")
        print(prototype)
        print("")
        label_surface = FONTS['font_14_bold'].render(prototype['name'], 1, (0xE4, 0x88, 0x20))
        yy += 19
        DISPLAY.blit(label_surface, (484, 110 + yy))
    print("=== /Prototypes ===")
    print("")



    build_queue = colony.build_queue()
    yy = 0
    print("")
    print("=== Build Queue: ===")
    for queue_item in build_queue:
        print(queue_item)
        build_id = queue_item['item']
        if build_id <  255:
            if build_id == 150:
                label = PROTOTYPES[0]['name']
            elif build_id == 148:
                label = PROTOTYPES[1]['name']
            elif build_id == 147:
                label = PROTOTYPES[2]['name']
            elif build_id == 145:
                label = PROTOTYPES[3]['name']
            elif build_id == 144:
                label = PROTOTYPES[4]['name']
            else:
                label = BUILDINGS[build_id]['name']

            label_surface = FONTS['font_12_bold'].render(label, 1, (0xE4, 0x88, 0x20))
            xx = label_surface.get_width() // 2
            DISPLAY.blit(label_surface, (208 + 126 - xx, 332 + yy))
            yy += 20
    print("=== /Build Queue: ===")
    print("")

    pygame.display.flip()



# end func draw

#
#       RUN
#
def run(GAME, colony_id):

    triggers = [
        {'action': "ESCAPE",        'rect': pygame.Rect((496, 448), (56, 16))}
    ]

    DATA	= GAME['DATA']

    colony      = DATA['colonies'][colony_id]

    planet_id	= colony.get_planet_id()
    planet      = DATA['planets'][planet_id]

    star_id	= planet.get_star()
    star	= DATA['stars'][star_id]

    draw(GAME, star, planet, colony)

    while True:
        event = get_input.get_input(triggers)
	action = event['action']
	
        if (action == "ESCAPE"):
            return
        
        else:
            print "UNKNONW ACTION: " + action






__author__="peterman"
__date__ ="$Jan 7, 2009 8:44:49 PM$"

import pygame
import get_input

import load_game

##
#	DRAW
##

def draw(GAME):

    DISPLAY	= GAME['DISPLAY']
    IMAGES	= GAME['IMAGES']
    
    DISPLAY.blit(IMAGES['GAME_MENU']['main'], (144, 25))

    buttons = []

    pygame.display.flip()
    return buttons
# end func draw

def run(GAME):

    print "### game_menu::run()"

    triggers = [
        {'action': "ESCAPE",           'rect': pygame.Rect((370, 342), (64, 17))},
        {'action': "load_game",         'rect': pygame.Rect((294, 71), (86, 23))}
    ]

    keys = {
	'KEYDOWN:108':          "load_game",        # L
	'KEYDOWN:115':          "save_game"         # S
    }

    draw(GAME)

    while True:
        event = get_input.get_input(triggers)
        action = event['action']

	if keys.has_key(action):
	    action = keys[action]

        if action == "ESCAPE":
            return
	elif action == "load_game":
	    load_game.run(GAME)
	    if GAME['close_game_menu']:
		GAME['close_game_menu'] = False
		return
	    else:
		draw(GAME)

        else:
            print "UNKNONW ACTION: " + action

# end func dialog_star_system



class AutoPlayer():

    def __init__(self, game_client):
	self.__game_client = game_client
	
    def play(self):
	save_file = self.__game_client.get_server_name()
	if save_file == "SAVE1.GAM":
	    print("AutoPlayer::play() ... starting AutoPlayer::play_SAVE1GAM()")
	    self.play_SAVE1GAM()
	else:
	    print("AutoPlayer::play() ... no scenario for %s" % save_file)

    def play_SAVE1GAM(self):
	cl = self.__game_client
	cl.set_research(75)		# Galactic Currency Exchange

	cl.next_turn()
	cl.next_turn()
	cl.next_turn()
	cl.next_turn()
	cl.next_turn()



#!/usr/bin/python

#import sys

from _research_areas import RESEARCH_AREAS
from _tech_table import TECH_TABLE
from _buildings import BUILDINGS


def main():
    """
        MAIN
    """
    for b_id in BUILDINGS:
        if b_id:
            b = BUILDINGS[b_id]
            if b['tech']:
                    tech_name = TECH_TABLE[b['tech']]['name']
                    if TECH_TABLE[b['tech']]['area']:
                        area_name = RESEARCH_AREAS[TECH_TABLE[b['tech']]['area']]['name']
            else:
                    tech_name = "???"
                    area_name = "???"
            print("%s : %s ... %s @ %s" % (str(b_id).rjust(3), b['name'].ljust(20), tech_name.ljust(30), area_name))

if __name__ == "__main__":
    main()



# To change this template, choose Tools | Templates
# and open the template in the editor.

__author__="peterman"
__date__ ="$May 15, 2010 12:52:51 PM$"

if __name__ == "__main__":
    print "Hello World";

class GameObject():

    def set_id(self, id):
        self.__id = id

    def get_id(self):
        return self.__id

    def set_name(self, name):
        self.__name = name

    def get_name(self):
        return self.__name

class SpaceObject(GameObject):

    def set_x(self, x):
        self.__x = x

    def set_y(self, y):
        self.__y = y

    def get_x(self):
        return self.__x

    def get_y(self):
        return self.__y



__author__="peterman"
__date__ ="$Jan 7, 2009 8:44:49 PM$"

import pygame
from pygame.locals import *

MOUSE_LEFT_BUTTON       = 1
MOUSE_MIDDLE_BUTTON     = 2
MOUSE_RIGHT_BUTTON      = 3
MOUSE_WHEELUP		= 4
MOUSE_WHEELDOWN		= 5

def get_input(triggers):

    tmpX, tmpY = 0, 0
    while True:

        events = pygame.event.get()

        for event in events:

#            print event

            if (event.type == QUIT):
                return {'action': "QUIT"}

            elif (event.type == KEYDOWN):
                if (event.key == 27):
		    return {'action': "ESCAPE"}
		else:
		    return {'action': "key", 'key': event.key}

            elif (event.type == MOUSEBUTTONDOWN):

		if event.button == MOUSE_MIDDLE_BUTTON:
		    print event

		elif event.button == MOUSE_WHEELUP:
		    return {'action': "SCROLL_UP"}

		elif event.button == MOUSE_WHEELDOWN:
		    return {'action': "SCROLL_DOWN"}

		else:
            	    tmpX, tmpY = event.pos[0], event.pos[1]

            	    for trigger in triggers:
                	if trigger['rect'].collidepoint(event.pos):

                    	    if event.button == MOUSE_LEFT_BUTTON:
                        	return trigger

                    	    elif event.button == MOUSE_RIGHT_BUTTON:
#                        	return "help:" + button['action']
                        	return {'action': "help", 'help': trigger['action']}

            elif event.type == MOUSEMOTION:
                tmpX, tmpY = event.pos[0], event.pos[1]
                for trigger in triggers:
                    if trigger['rect'].collidepoint(event.pos):
                        return {'action': "hover", 'hover': trigger}


__author__="peterman"
__date__ ="$Apr 10, 2010 9:25:30 PM$"

from setuptools import setup,find_packages

setup (
  name = 'taurus',
  version = '0.1',
  packages = find_packages(),

  # Declare your packages' dependencies here, for eg:
  install_requires=['foo>=3'],

  # Fill in these to make your Egg ready for upload to
  # PyPI
  author = 'peterman',
  author_email = '',

  summary = 'Just another Python package for the cheese shop',
  url = '',
  license = '',
  long_description= 'Long description of the package',

  # could also include long_description, download_url, classifiers, etc.

  
)



#from math import floor

from _game_constants import *

from _buildings import *

GOVERMENT_FEUDAL	= 0
GOVERMENT_DICTATORSHIP	= 2
GOVERMENT_DEMOCRACY	= 4
GOVERMENT_UNIFICATION	= 6

TERRAIN_TOXIC		= 0
TERRAIN_RADIATED	= 1
TERRAIN_BARED		= 2
TERRAIN_DESERT		= 3
TERRAIN_TUNDRA		= 4
TERRAIN_OCEAN		= 5
TERRAIN_SWAMP		= 6  
TERRAIN_ARID		= 7
TERRAIN_TERRAN		= 8
TERRAIN_GAIA		= 9

MINERAL_ULTRAPOOR	= 0
MINERAL_POOR		= 1
MINERAL_ABUNDANT	= 2
MINERAL_RICH		= 3
MINERAL_ULTRARICH	= 4

SPECIAL_ARTIFACTS	= 10

R_MICROLITE_CONSTRUCTION	= 108

def get_colony_farming(planet, colony, players):
    if colony.is_outpost():
	return 0

    planet_terrain = planet['terrain']
    planet_bases = [0, 0, 0, 2, 2, 2, 2, 4, 4, 6]
    farming = 0

#    print "	planet_terrain: %s" % planet['terrain']

    colony_owner = colony.owner
    player_goverment = players[colony_owner]['racepicks']['goverment']
#    print "	player_goverment: %i" % player_goverment

    for colonist in colony.colonists[FARMER]:
	racepick_farming = players[colonist['race']]['racepicks']['farming']
	racepick_aquatic = players[colonist['race']]['racepicks']['aquatic']
#	print "	colonist: %s ... %i (aquatic = %i)" % (colonist, racepick_farming, racepick_aquatic)

	if racepick_aquatic and (planet_terrain in (TERRAIN_OCEAN, TERRAIN_TERRAN)):
	    base = planet_bases[TERRAIN_GAIA]
	elif racepick_aquatic and (planet_terrain in (TERRAIN_TUNDRA, TERRAIN_SWAMP)):
	    base = planet_bases[TERRAIN_TERRAN]
	else:
	    base = planet_bases[planet_terrain]
	
	
	farming += base + racepick_farming

    if player_goverment == GOVERMENT_UNIFICATION:
	farming = (farming * 150) / 100
    
    return farming / 2
# end func get_colony_farming




__author__="peterman"
__date__ ="$Jan 7, 2009 8:44:49 PM$"

import pygame
import get_input

from datetime import datetime

import savegame

##
#	DRAW
##

def draw(GAME):
    from game import stardate
    print "### load_game::draw()"

    DISPLAY	= GAME['DISPLAY']
    IMAGES	= GAME['IMAGES']
    FONTS	= GAME['FONTS']

    DISPLAY.blit(IMAGES['GAME_MENU']['load_game'], (144, 25))

    color_dark = (0xC8, 0x64, 0x08)
#    color_light = (0xFC, 0x88, 0x00)

    n = 0
    for sg in savegame.list_savegames():
	name = sg['name']
	date = datetime.fromtimestamp(sg['ST_MTIME']).strftime("%b %d, %Y %H:%M")
	y = 46 + (31 * n)
	DISPLAY.blit(FONTS['font_12'].render(name, 1, color_dark), (175, y))
	DISPLAY.blit(FONTS['font_10'].render("Stardate: %s" % stardate(sg['stardate']), 1, color_dark), (175, y + 14))
	DISPLAY.blit(FONTS['font_10'].render(date, 1, color_dark), (275, y + 14))
	n += 1

    buttons = []

    pygame.display.flip()
    return buttons
# end func draw

def run(GAME):

    print "### load_game::run()"

    triggers = [
        {'action': "ESCAPE",           'rect': pygame.Rect((318, 365), ( 63, 16))},
        {'action': "load:1",           'rect': pygame.Rect((168,  46), (232, 28))},
        {'action': "load:2",           'rect': pygame.Rect((168,  77), (232, 28))},
        {'action': "load:3",           'rect': pygame.Rect((168, 108), (232, 28))},
        {'action': "load:4",           'rect': pygame.Rect((168, 139), (232, 28))},
        {'action': "load:5",           'rect': pygame.Rect((168, 170), (232, 28))},
        {'action': "load:6",           'rect': pygame.Rect((168, 201), (232, 28))},
        {'action': "load:7",           'rect': pygame.Rect((168, 232), (232, 28))},
        {'action': "load:8",           'rect': pygame.Rect((168, 263), (232, 28))},
        {'action': "load:9",           'rect': pygame.Rect((168, 294), (232, 28))},
        {'action': "load:10",           'rect': pygame.Rect((168, 325), (232, 28))}
    ]

    keys = {
	'KEYDOWN:49':          "load:1",        # 1
	'KEYDOWN:50':          "load:2",        # 2
	'KEYDOWN:51':          "load:3",        # 3
	'KEYDOWN:52':          "load:4",        # 4
	'KEYDOWN:53':          "load:5",        # 5
	'KEYDOWN:54':          "load:6",        # 6
	'KEYDOWN:55':          "load:7",        # 7
	'KEYDOWN:56':          "load:8",        # 8
	'KEYDOWN:57':          "load:9",        # 9
	'KEYDOWN:48':          "load:10",       # 0
    }

    draw(GAME)

    while True:
        event = get_input.get_input(triggers)
        action = event['action']

	if keys.has_key(action):
            action = keys[action]

        if action == "ESCAPE":
            return

	elif (action[:5] == "load:"):
	    filename = "SAVE%i.GAM" % int(action[5:])
	    print "Load game: %s" % filename
	    GAME['DATA'] = savegame.read_savegame(filename)
	    GAME['close_game_menu'] = True
	    return

        else:
            print "UNKNONW ACTION: " + action

# end func dialog_star_system



#!/usr/bin/python

import sys

from _research import RESEARCH_AREAS
from _tech_table import TECH_TABLE

def preview_area(area):
    while area:
	techs = ""
	for tech_id in TECH_TABLE:
	    tech = TECH_TABLE[tech_id]
	    if tech['area'] == area:
#		techs.append(tech['name'])
		techs += "%s : %s" % (str(tech_id).rjust(3), tech['name'].ljust(30))
#	print(" * %s" % techs)
#	print("" % (RESEARCH_AREAS[area]['name'], techs))
        print("%2i : %s (%s RP) :   %s " % (area, RESEARCH_AREAS[area]['name'].rjust(30), str(RESEARCH_AREAS[area]['cost']).rjust(5), techs))
	area = RESEARCH_AREAS[area]['next']
    print("")


def main(argv):
    """
        MAIN
    """
#    argc = len(argv)
#    if argc < 2:
#        print("Usage: %s <research_area>" % argv[0])
#        sys.exit(1)

#    area = int(argv[1])
    preview_area(4)
    preview_area(55)

    preview_area(22)
    preview_area(10)

    preview_area(28)
    preview_area(18)

    preview_area(57)
    preview_area(7)

if __name__ == "__main__":
    main(sys.argv)



SMALL_SHIP	= 0
MEDIUM_SHIP	= 1
LARGE_SHIP	= 2
HUGE_SHIP	= 3
TITAN_SHIP	= 4
DOOMSTAR_SHIP	= 5


// values for ship type field:
COMBAT_SHIP_TYPE	= 0
COLONY_SHIP_TYPE	= 1
TRANSPORT_SHIP_TYPE	= 2
#UNUSED                   3
OUTPOST_SHIP_TYPE	= 4



import socket
import pickle
#from socket import *
import game_socket

import time

#SOCKET_BUFFER_SIZE = 4096
#SOCKET_BUFFER_SIZE = 1024 * 64
#SOCKET_TIMEOUT = 1

__author__="peterman"
__date__ ="$Jan 1, 2010 2:42:50 PM$"

class GameClient():

    def __init__(self, host, port, buffer_size):
        self.__socket = game_socket.GameSocket(socket.socket(socket.AF_INET, socket.SOCK_STREAM))
        self.__socket.set_buffer_size(buffer_size)
        self.__socket.settimeout(1)
#        self.clientSocket = socket(AF_INET, SOCK_STREAM)
#        self.clientSocket.settimeout(SOCKET_TIMEOUT)
        self.__host = host
        self.__port = port
    # /__init__

    def connect(self):
        self.__socket.connect((self.__host, self.__port))
    # /connect

    def disconnect(self):
        self.__socket.close()
    # /disconnect

#    def recv(self):
#        return self.__socket.recv()

    def recv(self):
        return self.__socket.recv()

    def send(self, action, params = None):
	print("\n* ACTION: %s\nPARAMS: %s\n" % (action, str(params)))
	self.__socket.send({'action': action, 'params': params})
#        time.sleep(1)
    # /send

#    def send_data(self, data):
#	s = pickle.dumps(data)
#        print("* GameClient::send_data() ... %s" % s)
#        self.clientSocket.send(s)
#    # /send
    
    def __deprecated_recv(self):
        data = "";
        chunks = 0;
        while 1:
            try:
                s = self.recv()
#                s = self.clientSocket.recv(SOCKET_BUFFER_SIZE)
            except timeout:
                print("Game_client::recv ... self.clientSocket.recv > timeout")
                s = None
            if not s:
                break
            data += s
            chunks += 1
#        print("Received from server: " + data)
        print("GameClient::recv() ... Chunks received from game server: " + str(chunks))
        print("GameClient::recv() ... Bytes received from game server: " + str(len(data)))
        return data
    # /recv

    def __deprecated_recv_data(self):
        data = self.recv()
#        print("GameClient::recv_data() ... data = %s" % data[:20])
        if data[:9] == "DATA ... ":
            return pickle.loads(data[9:])
        else:
            return None
    # /recv_data

    def login(self, player_id):
#	print("GameClient::set_player_id() ... player_id = %s" % player_id)
	self.__player_id = player_id
        self.send("LOGIN", {'player_id': player_id})
    # /login

    def logout(self):
        self.send("LOGOUT")
    # /logout

    def get_server_name(self):
	self.send("GET_NAME")
	return self.recv()

    def fetch_game_data(self):
	self.send("FETCH_GAME_DATA")
        time.sleep(1)
        data = self.recv()
        if not data:
            print("! ERROR: GameClient::fetch_game_data() ... no data???")
        return data
    # /fetch_game_data

    def wait_for_next_turn(self):
#	print("GameClient::wait_for_next_turn()")
        self.send("NEXT_TURN")
        return True
    # /wait_for_next_turn

    def next_turn(self):
	if self.wait_for_next_turn():
 #   	    print("GameClient::next_turn()")
            response = self.recv()
            return response == "NEXT_TURN_ACK"
	else:
	    print("...FAILED")
    	    return False
    # /next_turn

    def set_research(self, tech_id):
#	print("GameClient::set_research() ... tech_id = %i" % tech_id)
        self.send("SET_RESEARCH", {'tech_id': tech_id})
	return self.fetch_game_data()
    # /set_research

    def get_server_status(self):
	self.send("GET_SERVER_STATUS")
	return self.recv()
    # /get_server_status

    def ping(self):
	tm1 = time.time()
	self.send("PING")
	ping_response = self.recv()
	tm2 = time.time()
	ping_time = round(tm2 - tm1, 2)
	if ping_response == "PONG":
	    print("PING: %ss" % (ping_time))
	else:
	    print("PING: WRONG RESPONSE '%s' returned in %ss" % (ping_response, ping_time))
    # /ping

"""
    def get_game_data(self):
        self.send("GET_GAME")
        return self.recv()

    def list_players(self):
        self.send("LIST_PLAYERS")
        return self.recv()

    def fetch_galaxy_info(self):
        self.send("GET_GALAXY_INFO")
        return self.recv()

    def list_stars(self):
        self.send("LIST_STARS")
        return self.recv()

    def list_planets(self):
        self.send("LIST_PLANETS")
        return self.recv()

    def list_colonies(self):
        self.send("LIST_COLONIES")
        return self.recv()
"""



#import os

import moo2
import star
import colony
import rules

def stardate(i):
    s = str(i)
    return s[:-1] + "." + s[-1]
# /stardate

class Game:

    def __init__(self, rules, game_file = None):
	self.set_recount_flag(False)
        self.set_rules(rules)
	if game_file:
	    self.load_moo2_savegame(game_file)
    # /__init__

    def set_recount_flag(self, value):
	self.__recount = value

    def set_rules(self, rules):
        self.__rules = rules
    # /set_rules

    def init_stars(self):
        self.__stars_by_coords = {}

        for star_id in self.__stars:
            star = self.__stars[star_id]
            k = "%i:%i" % (star.get_x(), star.get_y())
            self.__stars_by_coords[k] = star
            num = 0
            for object in star.get_objects():
#               print "object: %i" % object
                if object != 0xffff:
                    num += 1
                    self.__planets[object].set_position(num)
    # /init_stars
        
    def init_colonies(self):
        for colony_id in self.__colonies:
            colony	= self.__colonies[colony_id]
            planet_id	= colony.get_planet_id()
            if colony.get_owner() < 0xff:
                planet      = self.__planets[planet_id]
                colony.assign_planet(self.__planets[planet_id])
                colony.set_name("%s %i" % (self.__stars[planet.get_star()].get_name(), planet.get_position()))
    # /init_colonies

    def init_heroes(self):
        self.__players_heroes = {}
        for hero_id in self.__heroes:
            hero = self.__heroes[hero_id]
            if hero['player'] != 0xFF:
                if not self.__players_heroes.has_key(hero['player']):
                    self.__players_heroes[hero['player']] = {}
                self.__players_heroes[hero['player']][hero['id']] = hero

                

#        print self.__players_heroes
    # /init_heroes

    def load_moo2_savegame(self, filename):
        """
        Loads the original MOO2 savegame
        """

        savegame = moo2.Moo2Savegame(filename)

#        filesize = os.path.getsize(filename)
#        savefile = open(filename, 'rb')
#        data = savefile.read(filesize)
#        savefile.close()

        self.__game     = savegame.parse_game()
        self.__galaxy   = savegame.parse_galaxy()

        self.__heroes   = savegame.parse_heroes()          # 100%
        self.__players  = savegame.parse_players()
        self.__stars    = savegame.parse_stars()
        self.__planets	= savegame.parse_planets(self.__stars)		# 100%
        self.__colonies	= savegame.parse_colonies()
        self.__ships	= savegame.parse_ships()		# 100%

        self.init_stars()
        self.init_colonies()
        self.init_heroes()
	
	self.recount()
    # /load_moo2_savegame
 

    def get_colony_leader(self, colony_id):
#        print self.__colonies[colony_id].is_outpost()
        colony_owner = self.__colonies[colony_id].get_owner()
#        print
#        print("colony_id ... %i" % colony_id)
#        print("colony_data ... %s" % self.__colonies[colony_id])
#        print(self.__colonies[colony_id].planet())
#        print
        parent_star = self.__colonies[colony_id].planet().get_star()
 #       print "Game::get_colony_leader() ... colony_id = %i, owner = %i, parent_star = %i" % (colony_id, colony_owner, parent_star)
        for hero_id in self.list_player_colony_leaders(colony_owner):
            if self.__heroes[hero_id]['location'] == parent_star:
                return self.__heroes[hero_id]
        return None
    # /get_colony_leader

    def list_area_tech_ids(self, research_area):
        techs = []
        for tech_id in self.__rules['tech_table']:
            if self.__rules['tech_table'][tech_id]['area'] == research_area:
                techs.append(tech_id)
        return sorted(techs)
    # /list_techs_by_area


    def recount_heroes(self):
        for hero_id, hero in self.__heroes.iteritems():
            hero['level'] = 0
            for level in self.__rules['hero_levels']:
                if hero['experience'] >= level:
                    hero['level'] += 1
#            print self.__heroes[hero_id]

    def recount_colonies(self):
        for colony_id in self.__colonies:
            print("Game::recount_colonies() ... colony_id = %i" % colony_id)
            if self.__colonies[colony_id].exists():
#                print("Game::recount_colonies() ... self.__colonies[%i].owner() = %i" % (colony_id, self.__colonies[colony_id].owner()))
                colony_leader = self.get_colony_leader(colony_id)
#                print "Game::recount_colonies() ... colony_id %i, leader = %s" % (colony_id, str(colony_leader))
#                for hero_id in self.__heroes:
 #               print
                self.__colonies[colony_id].recount(self.__rules, colony_leader, self.__players)
    # /recount_colonies

    def list_player_colony_leaders(self, player_id):
        list = {}
        if not self.__players_heroes.has_key(player_id):
            return list
        for hero_id in self.__players_heroes[player_id]:
            hero = self.__heroes[hero_id]
#            print hero
            if hero['type'] == 1:
                list[hero_id] = hero
        return list
    # /list_player_colony_leaders

    def list_player_officers(self, player_id):
        list = {}
        if not self.__players_heroes.has_key(player_id):
            return list
        for hero_id in self.__players_heroes[player_id]:
            hero = self.__heroes[hero_id]
#            print hero
            if hero['type'] == 0:
                list[hero_id] = hero
        return list
    # /list_player_colony_leaders

    def update_research(self, player_id, research_item):
	print("Game::set_research() ... player_id = %i, research_item = %i" % (player_id, research_item))
	player = self.__players[player_id]
	player.set_research_item(research_item)
	player.set_research_area(self.__rules['tech_table'][research_item]['area'])
        player.set_research_costs(rules.research_costs(self.__rules['research_areas'], player.get_research_area(), player.get_research()))
        player.set_research_turns_left(rules.research_turns(player.get_research_costs(), player.get_research_progress(), player.get_research()))
	return True
    # /update_research

    def recount_players(self):
        for player_id in self.__players:
            self.__players[player_id].set_research(0)
            self.__players[player_id].set_food(0)


        for colony_id in self.__colonies:
            if self.__colonies[colony_id].exists():
                owner = self.__colonies[colony_id].get_owner()
                self.__players[owner].add_research(self.__colonies[colony_id].get_research())
                self.__players[owner].add_food(self.__colonies[colony_id].get_food() - self.__colonies[colony_id].total_population())

	# hardcoded 8 players
        for player_id in range(8):
            player = self.__players[player_id]

            if player.alive():
		self.update_research(player_id, player.get_research_item())
#                player.print_debug()

            # refresh player's research_areas:
            research_areas = {}
            for res_id in self.__rules['research']:
#                print "                         AREA: %s" % res_id
                area_id = self.__rules['research'][res_id]['start_area']
#                print "                         start_area = %i" % area_id
                while 1:
#                    print "      checking area_id = %i" % area_id

                    if not self.__rules['research_areas'][area_id]['next']:
                        break

                    area_techs = self.list_area_tech_ids(area_id)
#                    print "          area_techs = %s" % str(area_techs)
                    new_area_id = 0
                    for tech_id in area_techs:
                        if player.knows_technology(tech_id):
#                            print "known tech! ... %i .. moving to next area" % tech_id
                            new_area_id = self.__rules['research_areas'][area_id]['next']
                            break
                            
                    if new_area_id:
                        area_id = new_area_id
#                        player['research_levels'][res_id] = area_id
                    else:
                        break

                research_areas[res_id] = self.list_area_tech_ids(area_id)
            player.update_research_areas(research_areas)
#            print "                    research_areas = %s" % str(player['research_areas'])

#            for tech_id in player['known_techs']:
#                tech = self.__rules['tech_table'][tech_id]
#                print "                         known = %s (area: %i)" % (tech['name'], tech['area'])
            # / refresh player's research_areas
    # /recount_players

    def recount(self):
        print "=== Recount Heroes ==="
        self.recount_heroes()

        print "=== Recount Colonies ==="
        self.recount_colonies()

        print "=== Recount Players ==="
        self.recount_players()
    # /recount


    def raise_population(self):
        for colony_id in self.__colonies:
            self.__colonies[colony_id].raise_population()
    # /raise_population

    def get_stars_for_player(self, player_id):
        stars = {}
        for star_id in self.__stars:
#            print("star_id = %i" % star_id)
#            print(" visited = %i" % star['visited'])
            st = self.__stars[star_id]
            if st.visited_by_player(player_id):
                stars[star_id] = st
            else:
                # FIXME: this part returns dict instead of object!
                stars[star_id] = star.UnexploredStar(star_id, st.get_x(), st.get_y(), st.get_size(), st.get_pict_type(), st.get_class())
        return stars
    # /get_stars_for_player

    def get_colonies_for_player(self, player_id):
        colonies = {}
        for colony_id in self.__colonies:
            col = self.__colonies[colony_id]
            if col.get_owner() == player_id:
                colonies[colony_id] = col
            else:
                colonies[colony_id] = colony.EnemyColony(colony_id, col.get_owner())
        return colonies

    def get_data_for_player(self, player_id):
        """
        this method returns data for one particular player and leave data for other players
        security reasons to prevent hacked clients to display data that player should not know
        """

#        player_number = player_id + 1

        colony_leaders = self.list_player_colony_leaders(player_id)
#        print "=== Colony Leaders: ==="
#        print colony_leaders
#        print "=== /Colony Leaders: ==="

        officers = self.list_player_officers(player_id)
#        print "=== Officers: ==="
#        print officers
#        print "=== Officers: ==="


        return {
            'rules':            self.__rules,
            'me':               self.__players[player_id],

            'galaxy':           self.__galaxy,
            'players':          self.__players,                             # insecure
            'stars':            self.get_stars_for_player(player_id),       # 100% secure?
            'stars_by_coords':  self.__stars_by_coords,                     # insecure

            'colony_leaders':   colony_leaders,                             # 50% secure?
            'officers':         officers,                                   # 50% secure?

            'planets':          self.__planets,                             # insecure
            'colonies':         self.get_colonies_for_player(player_id),      # 100% secured ?
            'ships':            self.__ships,                               # insecure
            'prototypes':       self.__players[player_id].get_prototypes(),    # 100% secured?
        }
    # /get_data_for_player

    def next_turn(self):
#        raise research_progress
#        raise population

        print
        print "##"
        print "#    NEW TURN!"
        print "##"
        print

        self.recount()

        for player_id in self.__players:
            player = self.__players[player_id]
            if player.alive():

                # research:
                if player.get_research_costs() > 0:
                    player.raise_research()
                    if player.reseatch_completed():
                        print "research completed"
                        print player.get_known_techs()
                        print player.get_research_item()
                        player.add_known_technology(player.get_research_item())
#
                        research_area_id = player.get_research_area()
#
                        if research_area_id:
                            research_area = self.__rules['research_areas'][research_area_id]
                            if research_area['next']:
                                player.set_research_area(research_area['next'])

                        player.set_research_progress(-1)
                # /research

                # BC:
                self.__players[player_id].raise_bc()
                # /BC

        self.raise_population()

        self.recount()

        self.__galaxy['stardate'] += 1
    # /next_turn

    def count_players(self):
	c = 0
	for player_id in range(8):
	    if self.__players[player_id].alive():
		c += 1
	return c
    # /count_players

    def show_stars(self):
	s = ['small', 'medium', 'large']
	c = ['blue', 'white', 'yellow', 'orange', 'red', 'gray', 'black hole']
	print
	print("+---------+-----------------+------------+------------+--------+-------+-------+-------+-------+-------+")
	print("| star_id | name            | coords     | class      | size   | obj 1 | obj 2 | obj 3 | obj 4 | obj 5 |")
	print("+---------+-----------------+------------+------------+--------+-------+-------+-------+-------+-------+")
	for star_id, star in self.__stars.items():
	    objects = star.get_objects()
	    print("| %7i | %15s | %4i, %4i | %10s | %6s | %5i | %5i | %5i | %5i | %5i |" % (star_id, star.get_name(), star.get_x(), star.get_y(), c[star.get_class()], s[star.get_size()], objects[0], objects[1], objects[2], objects[3], objects[4]))
	print("+---------+-----------------+------------+------------+--------+-------+-------+-------+-------+-------+")
    # /show_stars

    def show_planets(self):
	si = ['tiny', 'small', 'medium', 'large', 'huge']
	ty = ["???", "asteroid belt", "gas giant", "planet"]
	mi = ['ultra poor', 'poor', 'abundant', 'rich', 'ultra rich']
	te = ['toxic', 'radiated', 'baren', 'desert', 'tundra', 'ocean', 'swamp', 'arid', 'terran', 'gaia', '?? K ??', '?? L ??']
	print
	print("+-----------+-----------------+----------+--------+---------------+------------+----------+------+---------+")
	print("| planet_id | star            | position | size   | type          | minerals   | terrain  | food | max pop |")
	print("+-----------+-----------------+----------+--------+---------------+------------+----------+------+---------+")
	for planet_id, planet in self.__planets.items():
	    print("| %9i | %15s | %8i | %6s | %13s | %10s | %8s | %4i | %7i |" % (planet_id, self.__stars[planet.get_star()].get_name(), planet.get_position(), si[planet.get_size()], ty[planet.get_type()], mi[planet.get_minerals()], te[planet.get_terrain()], planet.get_foodbase(), planet.get_max_population()))
	print("+-----------+-----------------+----------+--------+---------------+------------+----------+------+---------+")
	print
    # /show_planets

    def show_players(self):
	print("NUMBER OF PLAYERS: %i" % self.count_players())
	print("+-----------+----------------------+----------------------+--------+--------+--------+----------------------+----------------------+---------+-----------+")
	print("| player_id | name                 | emperor              | food   | BC     | income | research:area        | :item                | :costs  | :progress |")
	print("+-----------+----------------------+----------------------+--------+--------+--------+----------------------+----------------------+---------+-----------+")
	for player_id, player in self.__players.items():
	    print("| %9i | %20s | %20s | %6i | %6i | %6i | %20s | %20s | %7i | %9i |" % (player_id, player.get_race_name(), player.get_emperor_name(), player.get_food(), player.get_bc(), player.get_bc_income(), str(player.get_research_area()), str(player.get_research_item()), player.get_research_costs(), player.get_research_progress()))
	print("+-----------+----------------------+----------------------+--------+--------+--------+----------------------+----------------------+---------+-----------+")
	print
    # /show_players

    def show_colonies(self):
	print
	print("+-----------+-----------+-----------------+------------+------+----------+----------+-------+")
	print("| colony_id | planet_id | owner           | population | food | research | industry | BC    |")
	print("+-----------+-----------+-----------------+------------+------+----------+----------+-------+")
	for colony_id, colony in self.__colonies.items():
	    planet_id = colony.get_planet_id()
	    owner_id = colony.get_owner()
	    if (planet_id < 0xFF) and (owner_id < 0xFF):
		owner = self.__players[owner_id]
		print("| %9i | %9i | %15s | %10s | %4i | %8i | %8i | %5i |" % (colony_id, planet_id, owner.get_race_name(), colony.get_population(), colony.get_food(), colony.get_research(), colony.get_industry(), colony.bc()))
	    else:
		print("| %9i | %9s | %15s | %10s | %4i | %8i | %8i | ----- |" % (colony_id, "0xFF", "0xFF", colony.get_population(), colony.get_food(), colony.get_research(), colony.get_industry()))
#	    colony = self.colonies
	print("+-----------+-----------+-----------------+------------+------+----------+----------+-------+")
	print
    # /show_colonies
	

    def show_ships(self):
	print
	print("+---------+-----------------+----------+------------+-----------------+-------+-------+")
	print("| ship_id | owner           | status   | coords     | destination     | speed | turns |")
	print("+---------+-----------------+----------+------------+-----------------+-------+-------+")
	for ship_id, ship in self.__ships.items():
	    if ship.exists():
		print("| %7i | %15s | %8s | %4i, %4i | %15s | %5i | %5i |" % (ship_id, self.__players[ship.get_owner()].get_race_name(), ship.get_status_text(), ship.get_x(), ship.get_y(), self.__stars[ship.get_destination()].get_name(), ship.get_travelling_speed(), ship.get_turns_left()))
	print("+---------+-----------------+----------+------------+-----------------+-------+-------+")
	print



#!/usr/bin/python

import sys
import os

from stat import ST_MTIME
from time import sleep
#import savegame
import moo2

def debug_research(data):
    PLAYER_0    = data['players'][0]

    research_item = PLAYER_0['research_item']
    research_progress = PLAYER_0['research_progress']
    research_area = PLAYER_0['research_area']

    print("")
    print("=== Player #0: === ")

    print("known_techs: %s" % PLAYER_0['known_techs'])

    print("research_area: %i" % research_area)
    print("research_progress: %i" % research_progress)
    print("research_item: %i" % research_item)
    print("")

    print("    %s:    {" % str(research_item).rjust(3))
    print("        'name':     \"\",")
    print("        'area':     %i," % research_area)
    print("    },")
    print("")


    print("=== /Player #0 ===")
    print("")
    
def debug_player0_unknown(savegame):
    player = savegame.parse_players()[0]
    print("")
    print("=== Player #0: === ")
    print("0x28: %i" % player['0x28'])
    print("0x29: %i" % player['0x29'])
    print("0x2A: %i" % player['0x2A'])
    print("0x2B: %i" % player['0x2B'])
    print("0x2C: %i" % player['0x2C'])
    print("0x2D: %i" % player['0x2D'])
    print("0x2E: %i" % player['0x2E'])
    print("0x2F: %i" % player['0x2F'])

    print("")

    print("0x40: %i ... %i ... %i ...%i" % (player['0x40'], player['0x41'], player['0x42'], player['0x43']))
    print("0x44: %i ... %i ... %i ...%i" % (player['0x44'], player['0x45'], player['0x46'], player['0x47']))
    print("0x48: %i ... %i ... %i ...%i" % (player['0x48'], player['0x47'], player['0x4A'], player['0x4B']))
    print("0x4C: %i ... %i ... %i ...%i" % (player['0x4C'], player['0x4D'], player['0x4E'], player['0x4F']))
#    , player['0x44'], player['0x45'], player['0x46'], player['0x47'], player['0x48'])
#    print("0x49: %i" % player['0x49'], player['0x4A'], player['0x4B'], player['0x4C'], player['0x4D'], player['0x4E'], player['0x4F'])
    
    print("")

    print("0x50: %i ... %i ... %i ...%i" % (player['0x50'], player['0x51'], player['0x52'], player['0x53']))
    print("0x54: %i ... %i ... %i ...%i" % (player['0x54'], player['0x55'], player['0x56'], player['0x57']))
    print("0x58: %i ... %i ... %i ...%i" % (player['0x58'], player['0x57'], player['0x5A'], player['0x5B']))
    print("0x5C: %i ... %i ... %i ...%i" % (player['0x5C'], player['0x5D'], player['0x5E'], player['0x5F']))

    print("")

    print("0x60: %i ... %i ... %i ...%i" % (player['0x60'], player['0x61'], player['0x62'], player['0x63']))
    print("0x64: %i ... %i ... %i ...%i" % (player['0x64'], player['0x65'], player['0x66'], player['0x67']))
    print("0x68: %i ... %i ... %i ...%i" % (player['0x68'], player['0x67'], player['0x6A'], player['0x6B']))
    print("0x6C: %i ... %i ... %i ...%i" % (player['0x6C'], player['0x6D'], player['0x6E'], player['0x6F']))

    print("")

    print("0x70: %i ... %i ... %i ...%i" % (player['0x70'], player['0x71'], player['0x72'], player['0x73']))
    print("0x74: %i ... %i ... %i ...%i" % (player['0x74'], player['0x75'], player['0x76'], player['0x77']))
    print("0x78: %i ... %i ... %i ...%i" % (player['0x78'], player['0x77'], player['0x7A'], player['0x7B']))
    print("0x7C: %i ... %i ... %i ...%i" % (player['0x7C'], player['0x7D'], player['0x7E'], player['0x7F']))

    print("")

    print("0x80: %i ... %i ... %i ...%i" % (player['0x80'], player['0x81'], player['0x82'], player['0x83']))
    print("0x84: %i ... %i ... %i ...%i" % (player['0x84'], player['0x85'], player['0x86'], player['0x87']))
    print("0x88: %i ... %i ... %i ...%i" % (player['0x88'], player['0x87'], player['0x8A'], player['0x8B']))
    print("0x8C: %i ... %i ... %i ...%i" % (player['0x8C'], player['0x8D'], player['0x8E'], player['0x8F']))

    print("")

    print("0x90: %i ... %i ... %i ...%i" % (player['0x90'], player['0x91'], player['0x92'], player['0x93']))
    print("0x94: %i ... %i ... %i ...%i" % (player['0x94'], player['0x95'], player['0x96'], player['0x97']))
    print("0x98: %i ... %i ... %i ...%i" % (player['0x98'], player['0x97'], player['0x9A'], player['0x9B']))
    print("0x9C: %i ... %i ... %i ...%i" % (player['0x9C'], player['0x9D'], player['0x9E'], player['0x9F']))

    print("")

    print("0xA0: %i" % player['0xA0'])
    print("0xA1: %i" % player['0xA1'])
    print("0xA2: %i" % player['0xA2'])
    print("0xA3: %i" % player['0xA3'])
    print("0xA4: %i" % player['0xA4'])
    print("0xA5: %i" % player['0xA5'])
    print("0xA6: %i" % player['0xA6'])
    print("0xA7: %i" % player['0xA7'])
    print("0xA8: %i" % player['0xA8'])

    print("")

    print("0xAD: %i" % player['0xAE'])
    print("0xAE: %i" % player['0xAE'])

    print("")
    print("research_progress: %i" % player['research_progress'])
    print("research_area: %i" % player['research_area'])
   

def main(argv):
    """
        MAIN
    """
    argc = len(argv)
    if argc < 2:
        print("Usage: %s <savegame>" % argv[0])
        sys.exit(1)

    filename = "../moo2/%s" % argv[1]


    mtime1 = 0
    while 1:
        st = os.stat(filename)
        mtime2 = st[ST_MTIME]
        if mtime1 != mtime2:
            print("mtime: %i" % mtime2)
            GAME = moo2.Moo2Savegame(filename)
            debug_player0_unknown(GAME)
            mtime1 = mtime2
        else:
            sleep(1)





if __name__ == "__main__":
    main(sys.argv)




import copy

def parse_cli_args(argv, default_options = {}):
    argc = len(argv)
    options = copy.copy(default_options)
    params = []
    i = 1
    while i < argc:
        if argv[i][:1] == "-":
            options[argv[i]] = argv[i+1]
            i += 1
        else:
            params.append(argv[i])
        i += 1
    return (options, params)
# end func parse_cli_args 



import universe

__author__="peterman"
__date__ ="$May 16, 2010 1:24:40 PM$"

class Starship(universe.SpaceObject):

    def __init__(self, ship_id):
        self.set_id(ship_id)

    def set_design(self, design):
        self.__design = design

    def get_design(self):
        return self.__design

    def set_owner(self, player_id):
        self.__owner = player_id

    def get_owner(self):
        return self.__owner

    def set_status(self, status):
        self.__status = status

    def get_status(self):
        return self.__status

    def exists(self):
	return self.get_status() != 5

    def get_status_text(self):
	t = ["orbit", "travel", "launch", "???", "refit", "deleted", "build"]
	return t[self.get_status()]

    def is_orbiting(self):
        return self.get_status() == 0

    def is_travelling(self):
        return self.get_status() == 1

    def is_launching(self):
        return self.get_status() == 2

    def set_destination(self, destination):
        self.__destination = destination

    def get_destination(self):
        return self.__destination

    def set_group_has_navigator(self, group_has_navigator):
        self.__group_has_navigator = group_has_navigator

    def get_group_has_navigator(self):
        return self.__group_has_navigator

    def set_travelling_speed(self, travelling_speed):
        self.__travelling_speed = travelling_speed

    def get_travelling_speed(self):
        return self.__travelling_speed

    def set_turns_left(self, turns_left):
        self.__turns_left = turns_left

    def get_turns_left(self):
        return self.__turns_left

    def set_shield_damage_percent(self, shield_damage_percent):
        self.__shield_damage_percent = shield_damage_percent

    def get_shield_damage_percent(self):
        return self.__shield_damage_percent

    def set_drive_damage_percent(self, drive_damage_percent):
        self.__drive_damage_percent = drive_damage_percent

    def get_drive_damage_percent(self):
        return self.__drive_damage_percent

    def set_computer_damage(self, computer_damage):
        self.__computer_damage = computer_damage

    def get_computer_damage(self):
        return self.__computer_damage

    def set_crew_quality(self, crew_quality):
        self.__crew_quality = crew_quality

    def get_crew_quality(self):
        return self.__crew_quality

    def set_crew_experience(self, crew_experience):
        self.__crew_experience = crew_experience

    def get_crew_experience(self):
        return self.__crew_experience

    def set_officer_id(self, officer_id):
        self.__officer_id = officer_id

    def get_officer_id(self):
        return self.__officer_id

    def set_special_device_damage(self, special_device_damage):
        self.__special_device_damage = special_device_damage

    def get_special_device_damage(self):
        return self.__special_device_damage

    def set_armor_damage(self, armor_damage):
        self.__armor_damage = armor_damage

    def get_armor_damage(self):
        return self.__armor_damage

    def set_structural_damage(self, structural_damage):
        self.__structural_damage = structural_damage

    def get_structural_damage(self):
        return self.__structural_damage

    def set_mission(self, mission):
        self.__mission = mission

    def get_mission(self):
        return self.__mission

    def set_just_built(self, just_built):
        self.__just_built = just_built

    def get_just_built(self):
        return self.__just_built

    def print_debug(self, owner_player, stars):
        print
        print("=== ship # %i ... %s ===" % (self.get_id(), owner_player.get_race_name()))
	print("    status = %i" % self.get_status())
        print("    coords = %i, %i" % (self.get_x(), self.get_y()))
#	print("    group_has_navigator = %i" % self.get_group_has_navigator())
#	print("    missin = %i" % self.get_mission())
        print("    destination = %i" % (self.get_destination()))
        print("    turns_left = %i" % self.get_turns_left())
#        print("     star: %s" % stars[self.get_location()].get_name())
#        print("     star: %s" % stars[self.get_location_x()].get_name())

#        for star_id in stars:
#            print("star # %i ... %s" % (star_id, stars[star_id].get_name()))
#        print("/ship")
        print



import universe
import lbx

__author__="peterman"
__date__ ="$May 16, 2010 11:00:06 AM$"

class Planet(universe.GameObject):

    def __init__(self, planet_id):
        self.set_id(planet_id)

    def set_colony_id(self, colony_id):
        self.__colony_id = colony_id

    def get_colony_id(self):
        return self.__colony_id

    def set_star(self, star_id):
        self.__star_id = star_id

    def get_star(self):
        return self.__star_id

    def set_position(self, position):
        self.__position = position

    def get_position(self):
        return self.__position

    def set_type(self, type):
        self.__type = type

    def get_type(self):
        return self.__type

    def set_size(self, size):
        self.__size = size

    def get_size(self):
        return self.__size

    def set_gravity(self, gravity):
        self.__gravity = gravity

    def get_gravity(self):
        return self.__gravity

    def set_terrain(self, terrain):
        self.__terrain = terrain

    def get_terrain(self):
        return self.__terrain

    def set_picture(self, picture):
        self.__picture = picture

    def get_picture(self):
        return self.__picture

    def set_minerals(self, minerals):
        self.__minerals = minerals

    def get_minerals(self):
        return self.__minerals

    def set_foodbase(self, foodbase):
        self.__foodbase = foodbase
        
    def get_foodbase(self):
        return self.__foodbase

    def set_terraformations(self, terraformations):
        self.__terraformations = terraformations

    def get_terraformations(self):
        return self.__terraformations

    def set_max_farms(self, max_farms):
        self.__max_farms = max_farms

    def get_max_farms(self):
        return self.__max_farms

    def set_max_population(self, max_population):
        self.__max_population = max_population

    def get_max_population(self):
        return self.__max_population

    def set_special(self, special):
        self.__special = special

    def get_special(self):
        return self.__special

    def set_flags(self, flags):
        self.__flags = flags

    def get_flags(self):
        return self.__flags

    def import_from_moo2(self, data):
        self.set_colony_id(lbx.read_short_int(data, 0x00))	# 0xffff = no colony here
        self.set_star(lbx.read_byte(data, 0x02))
        self.set_position(lbx.read_byte(data, 0x03))
        self.set_type(lbx.read_byte(data, 0x04))
        self.set_size(lbx.read_byte(data, 0x05))
        self.set_gravity(lbx.read_byte(data, 0x06))
#        self.set_group(lbx.read_byte(data, 0x07))   # not used ?
        self.set_terrain(lbx.read_byte(data, 0x08))
        self.set_picture(lbx.read_byte(data, 0x09))   # Background image on colony screen (0-5=image in planets.lbx)
        self.set_minerals(lbx.read_byte(data, 0x0a))
        self.set_foodbase(lbx.read_byte(data, 0x0b))
        self.set_terraformations(lbx.read_byte(data, 0x0c))
        self.set_max_farms(lbx.read_byte(data, 0x0d))   # unknown (Initial value is based on Planet Size but changes if colonized), 2=tiny, 4=small, 5=med, 7=large, A=huge
        self.set_max_population(lbx.read_byte(data, 0x0e))
        self.set_special(lbx.read_byte(data, 0x0f))
        self.set_flags(lbx.read_byte(data, 0x10))    # (bit 2 = Soil Enrichment)

    def is_asteroid_belt(self):
        return self.get_type() == 1

    def is_gas_giant(self):
        return self.get_type() == 2

    def is_planet(self):
        return self.get_type() == 3




__author__="peterman"
__date__ ="$Jan 7, 2009 8:44:49 PM$"

import pygame
import get_input

import colony_screen
import main_screen

PLANET_TYPE_ASTEROIDS   = 1
PLANET_TYPE_GAS_GIANT   = 2
PLANET_TYPE_PLANET      = 3

COLONY_TYPE_COLONY      = 0
COLONY_TYPE_OUTPOST     = 1

##
#	DRAW
##

def draw(GAME, star_id):

    DISPLAY	= GAME['DISPLAY']
    IMAGES	= GAME['IMAGES']
    FONTS	= GAME['FONTS']
    DICTIONARY	= GAME['DICTIONARY']
    DATA	= GAME['DATA']
    ME          = DATA['me']

    buttons = []

    X, Y = 106, 103
    star = DATA['stars'][star_id]

    print()
    print("@@@ dialog_systemDetail")
    print("     Star: #%i ... %s" % (star_id, star.get_name()))
    print("         Visited: %i" % (star.visited()))
    print()

# dialog window
    DISPLAY.blit(IMAGES['STARSYSTEM_DIALOG'], (X, Y))
# dialog title
    title = "Star System " + star.get_name()
    (tw, th) = FONTS['FONT3'].size(title)

#20284c
#    print star.get_objects()
#    for i in [4, 3, 2, 1, 0]:
    if not star.visited():
        DISPLAY.blit(FONTS['FONT3'].render(title, 1, (0x66, 0x66, 0x66)), (X + 174 - (tw / 2), Y + 14))
    else:
        DISPLAY.blit(FONTS['FONT3'].render(title, 1, (0x20, 0x28, 0x4c)), (X + 175 - (tw / 2), Y + 15))
        DISPLAY.blit(FONTS['FONT3'].render(title, 1, (0x50, 0x6c, 0x90)), (X + 174 - (tw / 2), Y + 14))
        for i in range(5):
            planet_id = star.get_objects()[i]
            print " object_id #%i" % planet_id
            if planet_id != 0xffff:
#               orbit_img_key = "STARSYSTEM_ORBIT%i" % i
                planet = DATA['planets'][planet_id]
                print "		type: %s" % (DICTIONARY['PLANET_TYPES'][planet.get_type()])
                print "		size: %s" % (DICTIONARY['PLANET_SIZES'][planet.get_size()])
                if planet.is_asteroid_belt():
                    if i == 0:
                        DISPLAY.blit(IMAGES['SYSTEM_ASTEROIDS'][i], (X + 30, Y + 60))
                elif planet.is_gas_giant():
                    DISPLAY.blit(IMAGES['SYSTEM_ORBIT'][i], (X + 30, Y + 60))
                elif planet.is_planet():
                    DISPLAY.blit(IMAGES['SYSTEM_ORBIT'][i], (X + 30, Y + 60))
                else:
                    print("WARNING: unknow star system object detected: %i" % (planet.get_type()))

        for i in range(5):
            planet_id = star.get_objects()[i]
            print " object_id #%i" % planet_id
            if planet_id != 0xffff:
                planet = DATA['planets'][planet_id]
                if planet.is_gas_giant():
                    DISPLAY.blit(IMAGES['SYSTEM_GAS_GIANTS'][planet.get_size()], (X + 30 + (60 * i), Y + 60))
                elif planet.is_planet():
#                    terrain = planet['terrain']
#                   print "		name: %s %i" % (planet['parent_star']['name'], i + 1)
#                    print "		gravity: %s" % (DICTIONARY['PLANET_GRAVITIES'][planet.get_gravity()])
 #                   print "		terran: %s" % (DICTIONARY['PLANET_TERRAINS'][planet.get_terrain()])
                    x = X + 30 + (60 * i)
                    y = Y + 60
                    DISPLAY.blit(IMAGES['SYSTEM_PLANETS'][planet.get_terrain()][planet.get_size()], (x, y))
                    colony_id = planet.get_colony_id()
                    if colony_id < 0xffff:
                        colony = DATA['colonies'][colony_id]
#                        print "         colony_id #%i" % colony_id
                        player = DATA['players'][colony.get_owner()]

                        if colony.is_owned_by(ME.get_id()) and colony.is_colony():
 #                           print("colony.population() = %i" % colony.population())
                            pop = str(colony.get_population())
                            buttons.append({'action': "colony", 'colony_id': colony_id, 'rect': pygame.Rect((x, y), (31, 31))})
                        else:
                            pop = ""
                            
                        if colony.is_outpost():
                            DISPLAY.blit(IMAGES['SYSTEM_OUTPOST_MARKS'][player.get_color()], (x - 6, y))
                        else:
# colony['type'] == COLONY_TYPE_COLONY:
                            DISPLAY.blit(IMAGES['SYSTEM_COLONY_MARKS'][player.get_color()], (x - 6, y))
                            DISPLAY.blit(FONTS['font_12'].render(pop, 1, (0xff, 0xff, 0xff)), (x+5, y+5))

#                        print planet
#                        print colony
        DISPLAY.blit(IMAGES['SYSTEM_STAR'][star.get_class()], (X + 160, Y + 120))


#    for terrain in range(10):
#        for size in range(5):
    pygame.display.flip()
    return buttons
# end func draw

def run(GAME, star_id):

    triggers = [
        {'action': "ESCAPE",           'rect': pygame.Rect((370, 342), (64, 17))}
    ] + draw(GAME, star_id)

    while True:
        event = get_input.get_input(triggers)
        action = event['action']

        if action == "ESCAPE":
            return

        elif (action == "colony"):
            colony_screen.run(GAME, event['colony_id'])
            main_screen.draw(GAME)
            draw(GAME, star_id)

        else:
            print "UNKNONW ACTION: " + action

# / end func dialog_star_system



__author__="peterman"
__date__ ="$Jan 7, 2009 8:44:49 PM$"

if __name__ != "__main__":
    exit

import sys
import pygame

import get_input

from game import stardate

import colonies_screen
import leaders_screen
import dialog_star_system
import game_menu

import info_screen
import research_screen
import planets_screen

import debug

ZOOM_LEVEL = None
map_x, map_y, map_width, map_height = 22, 21, 505, 400

##
#	DRAW
##

def draw_ships(GAME):
    global ZOOM_LEVEL
    global map_x, map_y, map_width, map_height

    PLAYERS = GAME['DATA']['players']
    STARS = GAME['DATA']['stars']
    SHIPS = GAME['DATA']['ships']
    STARS_BY_COORDS = GAME['DATA']['stars_by_coords']

    DISPLAY	= GAME['DISPLAY']
    IMAGES	= GAME['IMAGES']

#    PLAYERS		= GAME['DATA']['players']
#    STARS		= GAME['DATA']['stars']
#    STARS_BY_COORDS	= GAME['DATA']['stars_by_coordinates']
#    SHIPS		= GAME['DATA']['ships']

    galaxy_width, galaxy_height = GAME['DATA']['galaxy']['width'], GAME['DATA']['galaxy']['height']

    for ship_id in SHIPS:
	draw = False
        ship = SHIPS[ship_id]
    	x = map_x + ((ship.get_x() * map_width) / galaxy_width)
    	y = map_y + ((ship.get_y() * map_height) / galaxy_height)
	ship_icon_size = ZOOM_LEVEL
	ship_destination = ship.get_destination()
        if ship.get_owner() < 8:
    	    ship_icon_color = PLAYERS[ship.get_owner()].get_color()
	else:
	    ship_icon_color = ship.get_owner()
	ship_icon = IMAGES['MAP_SHIPS'][ship_icon_color][ship_icon_size]

#        ship.print_debug(PLAYERS[ship.get_owner()], STARS)

	if ship.is_orbiting():
	    draw = True
	    star_class = STARS[ship_destination].get_class()
    	    star_pic_size = 2 - STARS[ship_destination].get_size() + ZOOM_LEVEL
	    spic_w, spic_h = IMAGES['MAP_STARS'][star_class][star_pic_size].get_size()
#	    x += 8
#	    y -= 13
	    x += spic_w / 3 - 1 - ZOOM_LEVEL
	    y -= spic_h / 2
        elif ship.is_travelling():
	    draw = True
	    ship_w, ship_h = ship_icon.get_size()
	    x -= (ship_w / 2) - 4
	    y -= (ship_h / 2) - 2
        elif ship.is_launching():
	    draw = True
	    k = "%i:%i" % (ship.get_x(), ship.get_y())
	    star_class = STARS_BY_COORDS[k].get_class()
    	    star_pic_size = 2 - STARS_BY_COORDS[k].get_size() + ZOOM_LEVEL
	    spic_w, spic_h = IMAGES['MAP_STARS'][star_class][star_pic_size].get_size()
#    	    star_pic_size = 2 - STARS[ship['location']]['size'] + ZOOM_LEVEL
#	    spic_w, spic_h = IMAGES['MAP_STARS'][star_class][star_pic_size].get_size()
	    x -= (spic_w / 2) + 7
	    y -= (spic_h / 2) + 1
	if draw:
	    DISPLAY.blit(ship_icon, (x, y))
# end func draw_ships

def draw_stars(GAME):
    global ZOOM_LEVEL
    global map_x, map_y, map_width, map_height

    DATA        = GAME['DATA']
    GALAXY      = DATA['galaxy']

    galaxy_width, galaxy_height = GALAXY['width'], GALAXY['height']

    DISPLAY	= GAME['DISPLAY']
    FONTS	= GAME['FONTS']
    IMAGES	= GAME['IMAGES']
    STARS	= GAME['DATA']['stars']

    dynamic_buttons = []

    WORMHOLE_COLOR = 0x242428
#    WORMHOLE_COLOR = 0x663333

    wormholes = {}
    for star_id in STARS.keys():
        star = STARS[star_id]

        # draw the wormholes
        if star.visited():
            if star.wormhole() != 0xff:
                star2 = STARS[star.wormhole()]
                star_ids = [star.get_id(), star2.get_id()]
                key = (min(star_ids) << 8) + max(star_ids)
                if not wormholes.has_key(key):
                    x1, y1 = map_x + ((star.get_x() * map_width) / galaxy_width), map_y + ((star.get_y() * map_height) / galaxy_height)
                    x2, y2 = map_x + ((star2.get_x() * map_width) / galaxy_width), map_y + ((star2.get_y() * map_height) / galaxy_height)
                    wormholes[key] = True

                pygame.draw.line(DISPLAY, WORMHOLE_COLOR, (x1, y1), (x2, y2), 2)

    for star_id in STARS.keys():
        star = STARS[star_id]

        # draw the stars end their names
#    for star in STARS:
        star_class = star.get_class()
        star_size = 2 - star.get_size() + ZOOM_LEVEL
#        print "drawing star '%s' coordinates: [%i, %i], class: %i, size: %i" % (star.get_name(), star.get_x(), star.get_y(), star_class, star_size)

	pic_width = IMAGES['MAP_STARS'][star_class][star_size].get_width()
	pic_height = IMAGES['MAP_STARS'][star_class][star_size].get_height()
	
        x = map_x + ((star.get_x() * map_width) / galaxy_width)
        y = map_y + ((star.get_y() * map_height) / galaxy_height)

#        print " ... [%i, %i]" % (x + xx, y)
        if star_class < 7:
            xx = pic_width / 2
            yy = pic_height / 2
            dynamic_buttons.append({'action': "systemDetail:%i" % star.get_id(), 'rect': pygame.Rect((x - xx, y - yy), (pic_width, pic_height))})
            DISPLAY.blit(IMAGES['MAP_STARS'][star_class][star_size], (x- xx, y - yy))

    for star_id in STARS.keys():
        star = STARS[star_id]
        star_class = star.get_class()
        star_size = 2 - star.get_size() + ZOOM_LEVEL

        pic_width = IMAGES['MAP_STARS'][star_class][star_size].get_width()
        pic_height = IMAGES['MAP_STARS'][star_class][star_size].get_height()

        x = map_x + ((star.get_x() * map_width) / galaxy_width)
        y = map_y + ((star.get_y() * map_height) / galaxy_height)
        yy = pic_height / 2

#        xx = pic_width / 2
#        yy = pic_height / 2

        if star.visited():
            text_width, text_height = FONTS['font_12_bold'].size(star.get_name())
            DISPLAY.blit(FONTS['font_12_bold'].render(star.get_name(), 1, (0x99, 0x99, 0x99)), (x - (text_width / 2), y + yy - 3))
#        else:
#            DISPLAY.blit(FONTS['font_11'].render("BH", 1, (0x33, 0x33, 0x33)), (x - 6, y))

    return dynamic_buttons
# end func draw_stars

def draw(GAME):

    MOO2_DIR = "../moo2"

    ICON = pygame.image.load(MOO2_DIR + "/orion2-icon.png")
    pygame.display.set_icon(ICON)

    global ZOOM_LEVEL
    global map_x, map_y, map_width, map_height

    print("GAME.keys = %str" % GAME.keys())

    GUI         = GAME['GUI']

    DATA        = GAME['DATA']

    DISPLAY     = GAME['DISPLAY']
    IMAGES      = GAME['IMAGES']
    FONTS       = GAME['FONTS']

#    RULES       = GAME['rules']

#    PLAYERS     = DATA['players']

    GALAXY      = DATA['galaxy']
#    STARS	= DATA['stars']
#    PLANETS	= DATA['planets']
#    COLONIES	= DATA['colonies']

    galaxy_width, galaxy_height = GALAXY['width'], GALAXY['height']
    galaxy_size_factor = GALAXY['size_factor']

    player		= DATA['me']

    DISPLAY.blit(IMAGES['MAIN_SCREEN']['background'], (0, 0))
    DISPLAY.blit(IMAGES['MAIN_SCREEN']['screen'], (0, 0))

    dynamic_buttons = []

    print "### main_screen::draw() ... galaxy_size_factor = %i" % galaxy_size_factor

#    if ZOOM_LEVEL is None:
    if galaxy_size_factor < 6:
	ZOOM_LEVEL = 4
    elif galaxy_size_factor < 11:
	ZOOM_LEVEL = 3
    elif galaxy_size_factor < 16:
	ZOOM_LEVEL = 2
    elif galaxy_size_factor < 21:
	ZOOM_LEVEL = 1
    else:
	ZOOM_LEVEL = 0
    print "### main_screen::draw() ... ZOOM_LEVEL = %i" % ZOOM_LEVEL

    dynamic_buttons += draw_stars(GAME)
    draw_ships(GAME)

# buzz-rectangle on the main screen star map to check transparent background
#    pygame.draw.rect(DISPLAY, 0x111111, ((map_x, map_y), (map_width, map_height)))

    DISPLAY.blit(FONTS['font_12'].render(stardate(DATA['galaxy']['stardate']), 1, (0xff, 0xff, 0xff)), (558, 27))
#    <Event(5-MouseButtonDown {'button': 2, 'pos': (561, 27)})>

    # BC
    if player.get_bc_income() < 0:
	bc_delta = "%i BC" % player.get_bc_income()
    else:
	bc_delta = "+%i BC" % player.get_bc_income()
    GUI.bordered_text(DISPLAY, "%i BC" % player.get_bc(), (0xff, 0xff, 0xff), (0x00, 0x00, 0x00), 554, 92, FONTS['font_12_bold'])
    GUI.bordered_text(DISPLAY, bc_delta, (0xff, 0xff, 0xff), (0x00, 0x00, 0x00), 555, 104, FONTS['font_12'])

    # food
    if player.get_food() < 0:
	food = "%i" % player.get_food()
    else:
	food = "+%i" % player.get_food()
    GUI.bordered_text(DISPLAY, food, (0xff, 0xff, 0xff), (0x00, 0x00, 0x00), 555, 250, FONTS['font_12'])

    # rp_spent
#    research_costs = 50
#    if player['research'] > 0:
#	research_turns = (50 - player['research_progress']) / player['research']
#    else:
#	research_turns = "~"

    # research
    GUI.bordered_text(DISPLAY, "~%s turns" % player.get_research_turns_left(), (0xff, 0xff, 0xff), (0x00, 0x00, 0x00), 552, 386, FONTS['font_11_bold'])
    GUI.bordered_text(DISPLAY, "%i RP" % player.get_research(), (0xff, 0xff, 0xff), (0x00, 0x00, 0x00), 555, 400, FONTS['font_12_bold'])

    pygame.display.flip()

#    debug.debug_players(GAME)
#    debug.debug_stars(GAME)
#    debug.debug_colonies_population(GAME)
#    debug.debug_ships(GAME)

    return dynamic_buttons
# end func draw

##
#	RUN
##

def run(GAME):

    GAME['DATA'] = GAME['client'].fetch_game_data()

    for key in GAME:
        print("GAME[%s] ... %s" % (key, type(GAME[key])))

    for key in GAME['DATA']:
        print("GAME['DATA'][%s] ... %s" % (key, type(GAME['DATA'][key])))


    GUI = GAME['GUI']

#    print(dir(GAME['DATA']))
#    GAME['DATA'] = next_turn(GAME['client'])
#    print(GAME['DATA']['players'][0])

    triggers = [
            {'action': "game_menu",		'rect': pygame.Rect((255,   8), (50, 12))},
            {'action': "colonies_screen",	'rect': pygame.Rect(( 20, 431), (65, 38))},
            {'action': "leaders_screen",	'rect': pygame.Rect((315, 431), (65, 38))},
            {'action': "info_screen",		'rect': pygame.Rect((460, 431), (65, 38))},
            {'action': "research_screen",	'rect': pygame.Rect((547, 347), (64, 64))},
            {'action': "newTurn",		'rect': pygame.Rect((547, 444), (59, 19))},
            {'action': "planets_screen",        'rect': pygame.Rect((93, 431), (65, 38))}
        ] + draw(GAME)

    GUI.set_image('COLSUM', GUI.load_image('COLSUM.LBX', 0, 0, 'COLSUM.LBX', None))

    keys = {
	'KEYDOWN:99':		"colonies_screen",	# C
	'KEYDOWN:103':		"game_menu",		# G
	'KEYDOWN:105':		"info_screen",		# I
	'KEYDOWN:108':		"leaders_screen",	# L
	'KEYDOWN:116':		"newTurn"		# T
    }

    player		= GAME['DATA']['me']

    while True:

        if player.get_research_progress() == -1:
            action = "research_screen"
            player.set_research_progress(0)
        else:
            event = get_input.get_input(triggers)
            action = event['action']

	"""
	    Keyboard input:
		c = Colonies
		l = Leaders
		t = New Turn
	"""
	if keys.has_key(action):
	    action = keys[action]

        if (action == "QUIT"):
            quit_game(GAME)

        elif (action == "newTurn"):
#	    print("=> newTurn ... DATA.keys = %s" % GAME['DATA'].keys())
#            GAME['DATA'] =
            if GAME['client'].next_turn():
                print("# NEXT_TURN succesfully sent")
                while True:
                    NEW_DATA = GAME['client'].fetch_game_data()
                    if NEW_DATA:
                        GAME['DATA'] = NEW_DATA
                        draw(GAME)
                        break
                    else:
                        print("ERROR: received None from GameClient::next_turn()")
            else:
                print("! ERROR: NEXT_TURN sent failed?")
#	    print("=> newTurn ... result = %s" % result)

        elif (action == "colonies_screen"):
            colonies_screen.run(GAME)
            draw(GAME)

        elif (action == "leaders_screen"):
            leaders_screen.run(GAME)
            draw(GAME)

        elif (action == "info_screen"):
            info_screen.run(GAME)
            draw(GAME)

        elif (action == "planets_screen"):
            planets_screen.run(GAME)
            draw(GAME)

        elif (action == "research_screen"):
            research_screen.run(GAME)
            draw(GAME)

        elif (action[:12] == "systemDetail"):
	    dialog_star_system.run(GAME, int(action[13:]))
            draw(GAME)

        elif (action == "game_menu"):
            game_menu.run(GAME)
            draw(GAME)

        else:
            print "UNKNONW ACTION: " + action

#  end func run


def quit_game(GAME):
    GAME['client'].logout()
    print
    print "@@@ quit_game"
    print "     turn: %i" % GAME['TURN']
    sys.exit(0)



FARMER		= 0x02
SCIENTIST	= 0x03
WORKER		= 0x82

GOVERMENT_FEUDAL        = 0
GOVERMENT_FEUDAL2       = 1
GOVERMENT_DICTATORSHIP  = 2
GOVERMENT_IMPERIUM	= 3
GOVERMENT_DEMOCRACY     = 4
GOVERMENT_UNIFICATION   = 6

PLANET_LOW_G	= 0
PLANET_NORMAL_G	= 1
PLANET_HEAVY_G	= 2

TERRAIN_TOXIC           = 0
TERRAIN_RADIATED        = 1
TERRAIN_BARED           = 2
TERRAIN_DESERT          = 3
TERRAIN_TUNDRA          = 4
TERRAIN_OCEAN           = 5
TERRAIN_SWAMP           = 6
TERRAIN_ARID            = 7
TERRAIN_TERRAN          = 8
TERRAIN_GAIA            = 9

MINERAL_ULTRAPOOR       = 0
MINERAL_POOR            = 1
MINERAL_ABUNDANT        = 2
MINERAL_RICH            = 3
MINERAL_ULTRARICH       = 4

SPECIAL_ARTIFACTS       = 10

BUILD_TRADE_GOODS	= 0xFE

TECH_ANTI_MISSILE_ROCKETS       =  13
TECH_COLONY_BASE                =  39
TECH_COLONY_SHIP                =  40
TECH_REINFORCED_HULL            =  56
TECH_BIOSPHERES                 =  61
TECH_FIGHTER_BAYS               =  66
TECH_FREIGHTERS                 =  69

TECH_HEIGHTENED_INTELLIGENCE	=  83
TECH_HYDROPONIC_FARM            =  87
TECH_TRANSPORT                  = 188
TECH_MICROLITE_CONSTRUCTION	= 107
TECH_OUTPOST_SHIP               = 108
TECH_NANO_DISASSEMBLERS		= 112
TECH_NEURAL_SCANNER             = 113
TECH_NUCLEAR_BOMB               = 119
TECH_PSIONICS                   = 146
TECH_SPACE_ACADEMY              = 163
TECH_SPY_NETWORK                = 165   # 166 ?
TECH_TELEPATHIC_TRAINING        = 181
TECH_VIRTUAL_REALITY_NETWORK	= 194

TECH_GROUP_CONSTRUCTIONS        =  1
TECH_GROUP_SPIES                =  2
TECH_GROUP_COLONY               =  3
TECH_GROUP_SHIP_EQUIPMENT       =  4
TECH_GROUP_GROUND_COMBAT        =  5
TECH_GROUP_MISCELANEOUS         =  6

TECH_REVIEW = {
    0: [],
    TECH_GROUP_CONSTRUCTIONS: [TECH_TRANSPORT, TECH_OUTPOST_SHIP, TECH_FREIGHTERS, TECH_COLONY_SHIP, TECH_COLONY_BASE]
}



import lbx
import universe

__author__="peterman"
__date__ ="$May 15, 2010 9:12:05 AM$"

class Star(universe.SpaceObject):

    def __init__(self, star_id):
        self.set_id(star_id)
    # /__init__

    def set_size(self, size):
        self.__size = size

    def get_size(self):
        return self.__size

    def set_pict_type(self, pict_type):
        self.__pict_type = pict_type

    def get_pict_type(self):
        return self.__pict_type

    def set_class(self, cl):
        self.__class = cl

    def get_class(self):
        return self.__class

    def set_objects(self, objects):
	self.__objects = objects

    def get_objects(self):
        return self.__objects

    def visited(self):
        return self.__visited

    def wormhole(self):
        return self.__wormhole

    def visited_by_player(self, player_id):
        return self.__visited & (1 << player_id)

    def import_from_moo2(self, data):

        self.__moo2data = data

        self.set_name(lbx.read_string(data, 0, 15))
        self.set_x(lbx.read_short_int(data, 0x0f))
        self.set_y(lbx.read_short_int(data, 0x11))
        self.set_size(lbx.read_byte(data, 0x13))
        self.__owner = lbx.read_byte(data, 0x14)		# primary owner
        self.set_pict_type(lbx.read_byte(data, 0x15))
        self.set_class(lbx.read_byte(data, 0x16))
        self.__last_planet_selected = [
                lbx.read_byte(data, 0x17),
                lbx.read_byte(data, 0x18),
                lbx.read_byte(data, 0x19),
                lbx.read_byte(data, 0x1A),
                lbx.read_byte(data, 0x1B),
                lbx.read_byte(data, 0x1C),
                lbx.read_byte(data, 0x1D),
                lbx.read_byte(data, 0x1E)
        ]

        self.__special = lbx.read_byte(data, 0x28)
        self.__wormhole = lbx.read_byte(data, 0x29)
        self.__visited = ord(data[0x33])	# bitmask as boleans for each player
        self.__indictor = ord(data[0x39])	# 0 = none, 1-8 = owner player 0-7  ( http://code.google.com/p/moo2x/wiki/star )
        self.__artemis = ord(data[0x3f])	# 0 = none, 1-8=owner player 0-7 ( http://code.google.com/p/moo2x/wiki/star )
        self.set_objects([
                lbx.read_short_int(data, 0x4a),
                lbx.read_short_int(data, 0x4c),
                lbx.read_short_int(data, 0x4e),
                lbx.read_short_int(data, 0x50),
                lbx.read_short_int(data, 0x52)
            ])
"""
                'black_hole_blocks':	[
                                            ],
                '0x1f':		ord(self.__data[0x1f]),
                '0x20':		ord(self.__data[0x20]),
                '0x21':		ord(self.__data[0x21]),
                '0x22':		ord(self.__data[0x22]),
                '0x23':		ord(self.__data[0x23]),
                '0x24':		ord(self.__data[0x24]),
                '0x25':		ord(self.__data[0x25]),
                '0x26':		ord(self.__data[0x26]),
                '0x27':		ord(self.__data[0x27]),
                '0x2a':		ord(self.__data[0x2a]),	# blockaded? ( 0 | 1 )
                '0x2b':		ord(self.__data[0x2b]),
                '0x2c':		ord(self.__data[0x2c]),
                '0x2d':		ord(self.__data[0x2d]),
                '0x2e':		ord(self.__data[0x2e]),
                '0x2f':		ord(self.__data[0x2f]),
                '0x30':		ord(self.__data[0x30]),
                '0x31':		ord(self.__data[0x31]),
                '0x32':		ord(self.__data[0x32]),
                '0x34':		ord(self.__data[0x34]),
                '0x35':		ord(self.__data[0x35]),
                '0x36':		ord(self.__data[0x36]),
                '0x37':		ord(self.__data[0x37]),
                '0x38':		ord(self.__data[0x38]),
                '0x3a':		ord(self.__data[0x3a]),
                '0x3b':		ord(self.__data[0x3b]),
                '0x3c':		ord(self.__data[0x3c]),
                '0x3d':		ord(self.__data[0x3d]),
                '0x3e':		ord(self.__data[0x3e]),
                '0x40':		ord(self.__data[0x40]),	# Star has dimensional portal ?
                '0x41':		ord(self.__data[0x41]),
                '0x42':		ord(self.__data[0x42]),
                '0x43':		ord(self.__data[0x43]),
                '0x44':		ord(self.__data[0x44]),
                '0x45':		ord(self.__data[0x45]),
                '0x46':		ord(self.__data[0x46]),
                '0x47':		ord(self.__data[0x47]),
                '0x48':		ord(self.__data[0x48]),
                '0x49':		ord(self.__data[0x49]),
    #		??? Relocation star id (0-7 player #, not sure about size of array. )
                '0x54':		ord(self.__data[0x54]),
                '0x55':		ord(self.__data[0x55]),
                '0x56':		ord(self.__data[0x56]),
                '0x57':		ord(self.__data[0x57]),
                '0x58':		ord(self.__data[0x58]),
                '0x59':		ord(self.__data[0x59]),
                '0x5a':		ord(self.__data[0x5a]),
                '0x5b':		ord(self.__data[0x5b]),
                '0x5c':		ord(self.__data[0x5c]),
                '0x5d':		ord(self.__data[0x5d]),
                '0x5e':		ord(self.__data[0x5e]),
                '0x5f':		ord(self.__data[0x5f]),
                '0x60':		ord(self.__data[0x60]),
                '0x61':		ord(self.__data[0x61]),
                '0x62':		ord(self.__data[0x62]),
                '0x63':		ord(self.__data[0x63]),
    #		unknown:
                '0x64':		ord(self.__data[0x64]),
                '0x65':		ord(self.__data[0x65]),
                '0x66':		ord(self.__data[0x66]),
                '0x67':		ord(self.__data[0x67]),
                '0x68':		ord(self.__data[0x68]),
                '0x69':		ord(self.__data[0x69]),
                '0x6a':		ord(self.__data[0x6a]),
                '0x6b':		ord(self.__data[0x6b]),
                '0x6c':		ord(self.__data[0x6c]),
                '0x6d':		ord(self.__data[0x6d]),
                '0x6e':		ord(self.__data[0x6e]),
                '0x6f':		ord(self.__data[0x6f]),
                '0x70':		ord(self.__data[0x70])
    # /import_from_moo2
"""

class UnexploredStar(Star):
    """
                stars[star_id] = {
                    'id':               star.get_id(),
                    'name':             "Unexplored",
                    'x':                star.get_x(),
                    'y':                star.get_y(),
                    'size':             star.size(),
                    'pict_type':        star.pict_type(),
                    'class':            star.get_class(),
                    'visited':          star.visited
                }
    """
    def __init__(self, star_id, x, y, size, pict_type, cl):
        self.set_id(star_id)
        self.set_name("Unexplored")
        self.set_x(x)
        self.set_y(y)
        self.set_size(size)
        self.set_class(cl)
        self.set_pict_type(pict_type)

    def visited(self):
        return False

    def visited_by_player(self):
        return False
    


import pygame

import get_input

#import formulas

from _game_constants import *

def print_summary_item(data, key, title):
    if data.has_key(key) and data[key]:
	print "+     % 6s ... % 25s    +" % (str(data[key]), title)

TECH_REVIEW = None

##
#       DRAW
##
#def draw(GAME, terrain, picture, colony_name):
def draw(GAME):
    from game import stardate
    DISPLAY     = GAME['DISPLAY']
    IMAGES      = GAME['IMAGES']
#    PALETTES	= GAME['PALETTES']
#    FONTS	= GAME['FONTS']

#    DICTIONARY	= GAME['DICTIONARY']

#    PLAYERS	= GAME['DATA']['players']

    DISPLAY.fill((0, 0, 0))
#    DISPLAY.fill((0x08, 0x38, 0x08))
    DISPLAY.blit(IMAGES['PLANETS_SCREEN']['screen'], (0, 0))

#    DISPLAY.blit(IMAGES['INFO_SCREEN']['return'], (535, 434))

    buttons = []

    pygame.display.flip()

    return buttons
# end func draw

##
#       RUN
##
def run(GAME):

    print
    print "### planets_screen::run()"

    triggers = [
        {'action': "ESCAPE",    		'rect': pygame.Rect((457, 444), ( 151, 16))},
    ]

    triggers2 = draw(GAME)

    while True:
        event = get_input.get_input(triggers + triggers2)
	action = event['action']

        if (action == "ESCAPE"):
            return

        else:
            print "UNKNONW ACTION: " + action

# end func run



import sys

import cli
import server
import game
import rules

def show_usage(name, message):
    print
    print(message)
    print("Usage:")
    print("	%s -g <savegame> [-h listen host] [-p listen port]" % name)
    print
# end func show_usage

def main(argv):
    """
        MAIN
    """

    default_options = {
	'-p':	9999,
	'-h':	"localhost",
        '-g':   "SAVE1.GAM"
    }

    (OPTIONS, PARAMS) = cli.parse_cli_args(argv, default_options)

    LISTEN_ADDR = OPTIONS['-h']
    LISTEN_PORT = OPTIONS['-p']
    GAME_FILE	= OPTIONS['-g']

    if GAME_FILE == "":
	show_usage(argv[0], "ERROR: Missing game file to load")
	sys.exit(1)

    print("* Init...")
    GAME = game.Game(rules.DEFAULT_RULES)
    print("* Load...")
    GAME.load_moo2_savegame(GAME_FILE)

#    GAME.show_stars()
#    GAME.show_planets()
    GAME.show_players()
#    GAME.show_colonies()
#    GAME.show_ships()

    SERVER = server.Server(LISTEN_ADDR, LISTEN_PORT, GAME)
    SERVER.set_name(GAME_FILE.split("/")[-1])

    print("* Run...")
    SERVER.run()

    print("* Exit...")
    sys.exit(0)

if __name__ == "__main__":
    main(sys.argv)
